package com.ruoyi.dashboard.index.domain.dto;

import java.time.LocalDate;
import java.util.Arrays;

import javax.validation.constraints.NotNull;

import org.springframework.format.annotation.DateTimeFormat;

import com.ruoyi.ss.payBill.domain.PayBillQuery;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.refund.domain.RefundQuery;
import com.ruoyi.ss.refund.domain.enums.RefundStatus;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

@Data
public class StoreDailyIncomeQuery {

    @ApiModelProperty("店铺ID")
    @NotNull(message = "店铺ID不能为空")
    private Long storeId;

    @ApiModelProperty("开始时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @NotNull(message = "开始时间不能为空")
    private LocalDate startDate;

    @ApiModelProperty("结束时间")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @NotNull(message = "结束时间不能为空")
    private LocalDate endDate;

    public PayBillQuery toPayBillQuery() {
        PayBillQuery billQuery = new PayBillQuery();
        billQuery.setStoreId(storeId);
        billQuery.setStatusList(PayBillStatus.payedList());
        billQuery.setPayDateStart(startDate);
        billQuery.setPayDateEnd(endDate);
        return billQuery;
    }

    public RefundQuery toRefundQuery() {
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreId(storeId);
        refundQuery.setCreateDateStart(startDate);
        refundQuery.setCreateDateEnd(endDate);
        refundQuery.setStatusList(RefundStatus.successList());
        return refundQuery;
    }

    public TransactionBillQuery toTransactionBillQuery() {
        TransactionBillQuery billQuery = new TransactionBillQuery();
        billQuery.setStoreId(storeId);
        billQuery.setStatusList(TransactionBillStatus.payedOrder());
        billQuery.setType(TransactionBillType.RECHARGE.getType());
        billQuery.setCreateDateRange(Arrays.asList(startDate, endDate));
        return billQuery;
    }
}
