package com.ruoyi.ss.channelWithdraw.service.impl;

import java.util.Collections;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.account.domain.AccountVO;
import com.ruoyi.ss.account.service.AccountService;
import com.ruoyi.ss.channelWithdraw.domain.ChannelWithdraw;
import com.ruoyi.ss.channelWithdraw.domain.ChannelWithdrawQuery;
import com.ruoyi.ss.channelWithdraw.domain.ChannelWithdrawVO;
import com.ruoyi.ss.channelWithdraw.domain.vo.ChannelWithdrawNameVO;
import com.ruoyi.ss.channelWithdraw.mapper.ChannelWithdrawMapper;
import com.ruoyi.ss.channelWithdraw.service.ChannelWithdrawService;

/**
 * 提现渠道Service业务层处理
 *
 * @author ruoyi
 * @date 2024-08-02
 */
@Service
public class ChannelWithdrawServiceImpl implements ChannelWithdrawService
{
    @Autowired
    private ChannelWithdrawMapper channelWithdrawMapper;

    @Autowired
    private AccountService accountService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询提现渠道
     *
     * @param channelId 提现渠道主键
     * @return 提现渠道
     */
    @Override
    public ChannelWithdrawVO selectChannelWithdrawByChannelId(Long channelId)
    {
        return channelWithdrawMapper.selectChannelWithdrawByChannelId(channelId);
    }

    /**
     * 查询提现渠道列表
     *
     * @param channelWithdraw 提现渠道
     * @return 提现渠道
     */
    @Override
    public List<ChannelWithdrawVO> selectChannelWithdrawList(ChannelWithdrawQuery channelWithdraw)
    {
        return channelWithdrawMapper.selectChannelWithdrawList(channelWithdraw);
    }

    /**
     * 新增提现渠道
     *
     * @param channelWithdraw 提现渠道
     * @return 结果
     */
    @Override
    public int insertChannelWithdraw(ChannelWithdraw channelWithdraw)
    {
        Integer result = transactionTemplate.execute(status -> {
            int insert = channelWithdrawMapper.insertChannelWithdraw(channelWithdraw);
            if (insert > 0) {
                clearCache();
            }
            return insert;
        });
        return result;
    }

    /**
     * 修改提现渠道
     *
     * @param channelWithdraw 提现渠道
     * @return 结果
     */
    @Override
    public int updateChannelWithdraw(ChannelWithdraw channelWithdraw)
    {
        int result = channelWithdrawMapper.updateChannelWithdraw(channelWithdraw);
        if (result > 0) {
            clearCache();
        }
        return result;
    }

    /**
     * 批量删除提现渠道
     *
     * @param channelIds 需要删除的提现渠道主键
     * @return 结果
     */
    @Override
    public int deleteChannelWithdrawByChannelIds(Long[] channelIds)
    {
        Integer result = transactionTemplate.execute(status -> {
            int delete = channelWithdrawMapper.deleteChannelWithdrawByChannelIds(channelIds);
            if (delete > 0) {
                clearCache();
            }
            return delete;
        });
        return result;
    }

    /**
     * 删除提现渠道信息
     *
     * @param channelId 提现渠道主键
     * @return 结果
     */
    @Override
    public int deleteChannelWithdrawByChannelId(Long channelId)
    {
        Integer result = transactionTemplate.execute(status -> {
            int delete = channelWithdrawMapper.deleteChannelWithdrawByChannelId(channelId);
            if (delete > 0) {
                clearCache();
            }
            return delete;
        });
        return result;
    }

    @Override
    public List<ChannelWithdrawVO> selectEnabledList(Long userId) {
        // 查询用户账户列表
        List<AccountVO> accountList = accountService.selectByUserId(userId);
        if (CollectionUtils.isEmptyElement(accountList)) {
            return Collections.emptyList();
        }
        // 查询渠道列表
        ChannelWithdrawQuery query = new ChannelWithdrawQuery();
        query.setAccountTypes(CollectionUtils.map(accountList, AccountVO::getAccountType));
        query.setEnabled(true);
        return this.selectChannelWithdrawList(query);
    }

    @Override
    public List<ChannelWithdrawVO> selectEnabledList() {
        // 查询渠道列表
        ChannelWithdrawQuery query = new ChannelWithdrawQuery();
        query.setEnabled(true);
        return this.selectChannelWithdrawList(query);
    }

    @Override
    public List<ChannelWithdrawNameVO> selectAllChannelNameList() {
        List<ChannelWithdrawNameVO> list = redisCache.getCacheList(CacheConstants.CHANNEL_WITHDRAW_NAME_LIST);
        if (CollectionUtils.isEmptyElement(list)) {
            list = channelWithdrawMapper.selectChannelNameList(new ChannelWithdrawQuery());
            if (CollectionUtils.isNotEmptyElement(list)) {
                redisCache.setCacheList(CacheConstants.CHANNEL_WITHDRAW_NAME_LIST, list);
            }
        }
        return list;
    }

    private void clearCache() {
        redisCache.deleteObject(CacheConstants.CHANNEL_WITHDRAW_NAME_LIST);
    }
}
