package com.ruoyi.ss.device.service.impl;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.ruoyi.ss.snLog.domain.SnLog;
import com.ruoyi.ss.transactionBill.service.TransactionAssembler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.enums.LoginType;
import com.ruoyi.common.enums.UserType;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.response.CommandResponse;
import com.ruoyi.iot.enums.IotHttpStatus;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.ss.commandLog.service.ICommandLogService;
import com.ruoyi.ss.device.domain.Device;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.DeviceCountVO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.dto.DeviceBatchUpdateModelDTO;
import com.ruoyi.ss.device.domain.dto.DeviceRegisterDTO;
import com.ruoyi.ss.device.domain.dto.DeviceWifiDTO;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.enums.DeviceServiceMode;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.domain.enums.DeviceUserOperaType;
import com.ruoyi.ss.device.domain.vo.DeviceMacSnVO;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.mapper.DeviceMapper;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.device.service.DeviceValidator;
import com.ruoyi.ss.device.utils.DeviceUtils;
import com.ruoyi.ss.deviceBindRecord.domain.enums.BindRecordType;
import com.ruoyi.ss.deviceBindRecord.domain.enums.BindRecordUserType;
import com.ruoyi.ss.deviceBindRecord.service.DeviceBindRecordService;
import com.ruoyi.ss.deviceSuit.service.DeviceSuitConverter;
import com.ruoyi.ss.deviceSuit.service.DeviceSuitService;
import com.ruoyi.ss.model.domain.SmModelVO;
import com.ruoyi.ss.model.domain.enums.ModelTag;
import com.ruoyi.ss.model.service.ModelService;
import com.ruoyi.ss.record.time.service.IRecordTimeService;
import com.ruoyi.ss.record.time.service.RecordTimeConverter;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.storeStaff.service.StoreStaffValidator;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.user.service.UserService;
import com.ruoyi.ss.user.service.UserValidator;

import lombok.extern.slf4j.Slf4j;

/**
 * 设备Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service
@Slf4j
public class DeviceServiceImpl implements DeviceService
{
    @Autowired
    private DeviceMapper deviceMapper;

    @Autowired
    private IotService iotService;

    @Autowired
    private DeviceBindRecordService smDeviceBindRecordService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private StoreService storeService;

    @Autowired
    private IRecordTimeService recordTimeService;

    @Autowired
    private RecordTimeConverter recordTimeConverter;

    @Autowired
    private UserService userService;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private DeviceSuitConverter deviceSuitConverter;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ModelService modelService;

    @Autowired
    private DeviceSuitService deviceSuitService;

    @Autowired
    private DeviceSuitConverter deviceConverter;

    @Autowired
    private UserValidator userValidator;

    @Autowired
    private StoreStaffValidator storeStaffValidator;

    @Autowired
    private ICommandLogService commandLogService;

    @Autowired
    private TransactionAssembler transactionAssembler;


    /**
     * 查询设备
     *
     * @param deviceId 设备主键
     * @return 设备
     */
    @Override
    public DeviceVO selectById(Long deviceId)
    {
        return deviceMapper.selectSmDeviceByDeviceId(deviceId);
    }

    /**
     * 查询设备列表
     *
     * @param smDevice 设备
     * @return 设备
     */
    @Override
    public List<DeviceVO> selectSmDeviceList(DeviceQuery smDevice) {
        return deviceMapper.selectSmDeviceList(smDevice);
    }

    /**
     * 新增设备
     * @param data 设备
     * @return 结果
     */
    @Override
    public int insertSmDevice(Device data) {
        ServiceUtil.assertion(data == null, "参数错误，不允许为空");
        ServiceUtil.assertion(StringUtils.isBlank(data.getDeviceNo()), "参数错误，SN不允许为空");
        ServiceUtil.assertion(StringUtils.isBlank(data.getMac()), "参数错误，MAC不允许为空");

        String key = data.getDeviceNo();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.ADD_DEVICE, key), "当前新增设备过于频繁，请稍后再试");
        try {

            SmModelVO model = modelService.selectSmModelByModelId(data.getModelId());
            ServiceUtil.assertion(model == null, "型号不存在");

            data.setCreateTime(DateUtils.getNowDate());
            data.setStatus(DeviceStatus.NORMAL.getStatus());
            data.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());

            Integer result = transactionTemplate.execute(status -> {
                // 创建设备
                int insert = deviceMapper.insertSmDevice(data);
                ServiceUtil.assertion(insert != 1, "新增设备失败");

                // 后校验
                DeviceVO vo = this.selectById(data.getDeviceId());
                deviceAssembler.assembleBonusList(vo);
                deviceValidator.afterCheck(vo);

                // 注册到OneNet
                this.registerToOneNet(vo);

                return insert;
            });
            return result == null ? 0 : result;
        } finally {
            redisLock.unlock(RedisLockKey.ADD_DEVICE, key);
        }
    }

    private void registerToOneNet(DeviceVO vo) {
        if (StringUtils.hasText(vo.getModelProductId())) {
            // 创建OneNet设备1
            int code = iotService.create(vo.getMac(), vo.getModelProductId());
            ServiceUtil.assertion(!IotHttpStatus.SUCCESS.equalCode(code) && !IotHttpStatus.DEVICE_EXIST.equalCode(code), "MAC-1注册失败");
            // 创建OneNet设备2
            if (StringUtils.hasText(vo.getMac2())) {
                code = iotService.create(vo.getMac2(), vo.getModelProductId());
                ServiceUtil.assertion(!IotHttpStatus.SUCCESS.equalCode(code) && !IotHttpStatus.DEVICE_EXIST.equalCode(code), "MAC-2注册失败");
            }
        }
    }

    /**
     * 修改设备
     *
     * @param data 设备
     * @return 结果
     */
    @Override
    public int updateSmDevice(DeviceBO data) {

        data.setUpdateTime(DateUtils.getNowDate());

        Integer result = transactionTemplate.execute(status -> {
       //     deviceMapper.selectByDeviceId(data.getDeviceNo());

            // 生成SnLog记录
            SnLog snLog = new SnLog();
            snLog.setDeviceId(data.getDeviceId());
            snLog.setUserId(SecurityUtils.getUserId());
            snLog.setMac(data.getMac());

            // 更新主表
            int update = deviceMapper.updateSmDevice(data);
            ServiceUtil.assertion(update != 1, "更新设备失败");



            // 后校验
            DeviceVO vo = this.selectById(data.getDeviceId());
            deviceAssembler.assembleBonusList(vo);
            deviceValidator.afterCheck(vo);

            // 若修改了MAC或者型号，则需要重新注册到OneNet
            if (data.getMac() != null || data.getMac2() != null || data.getModelId() != null) {
                this.registerToOneNet(vo);
            }

            // 更新套餐关联
            if (data.getSuitIds() != null) {
                // 删除设备套餐关联
                deviceSuitService.deleteByDeviceId(data.getDeviceId());
                // 新增关联
                deviceSuitService.batchInsert(deviceSuitConverter.toPo(data));
            }

            return update;
        });

        return result == null ? 0 : result;
    }

    /**
     * 批量删除设备
     *
     * @param deviceIds 需要删除的设备主键
     * @return 结果
     */
    @Override
    public int deleteSmDeviceByDeviceIds(Long[] deviceIds)
    {
        this.validatePreDelete(Arrays.asList(deviceIds));
        return deviceMapper.deleteSmDeviceByDeviceIds(deviceIds);
    }

    /**
     * 真正的删除校验
     */
    private void validatePreDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(true);
        Map<Long, DeviceVO> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            DeviceVO device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }
    }

    /**
     * 逻辑删除校验
     */
    private void validatePreLogicDelete(List<Long> deviceIds) {
        ServiceUtil.assertion(CollectionUtils.isEmpty(deviceIds), "设备不能为空");

        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(deviceIds);
        dto.setDeleted(false);
        // 前台用户，判断是不是他的设备
        if (LoginType.FRONT.equals(SecurityUtils.getLoginType())) {
            dto.setUserId(SecurityUtils.getUserId());
        }
        Map<Long, DeviceVO> deviceMap = selectMap(dto);

        for (Long deviceId : deviceIds) {
            DeviceVO device = deviceMap.get(deviceId);
            ServiceUtil.assertion(device == null, String.format("设备“%s”不存在，请刷新后重试", deviceId));
        }

    }

    /**
     * 条件查询列表，并转为Map
     */
    private Map<Long, DeviceVO> selectMap(DeviceQuery dto) {
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(DeviceVO::getDeviceId, item -> item));
    }

    @Override
    public int selectCount(DeviceQuery dto) {
        return deviceMapper.selectCount(dto);
    }

    @Override
    public List<DeviceVO> selectSmDeviceByStoreId(Long storeId) {
        if (storeId == null) {
            return Collections.emptyList();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setStoreId(storeId);
        return deviceMapper.selectSmDeviceList(dto);
    }

    @Override
    public int register(DeviceRegisterDTO dto) {
        if (dto == null) {
            return 0;
        }

        DeviceBO device = deviceConverter.toBOByRegister(dto);
        DeviceVO old = selectByAnyMac(dto.getMac());

        if (old == null) {
            // 若设备MAC不存在，则新增
            return this.insertSmDevice(device);
        } else {
            // 若设备MAC存在，则修改
            device.setDeviceId(old.getDeviceId());
            return this.updateSmDevice(device);
        }
    }

    @Override
    public List<DeviceMacSnVO> selectMacSnList(DeviceQuery query) {
        return deviceMapper.selectMacSnList(query);
    }

    @Override
    public DeviceVO selectOne(DeviceQuery query) {
        return deviceMapper.selectOne(query);
    }

    @Override
    public List<DeviceVO> selectUsingDevice(Long userId) {
        // 查询正在使用中的订单
        List<TransactionBillVO> usingBills = transactionBillService.selectUsingBill(userId);
        List<Long> deviceIds = usingBills.stream().map(TransactionBillVO::getDeviceId).distinct().collect(Collectors.toList());
        // 查询设备列表
        return this.selectByIds(deviceIds);
    }

    /**
     * 开关设备
     */
    @Override
    public int switchDevice(Long deviceId, boolean open, String reason) {
        if (deviceId == null) {
            return 0;
        }
        DeviceVO device = selectById(deviceId);
        if (device == null) {
            return 0;
        }
        this.pullDeviceInfo(device, null);
        return this.switchDevice(device, open ? DevicePowerStatus.ON : DevicePowerStatus.OFF, reason);
    }

    @Override
    public int batchUpdateModel(DeviceBatchUpdateModelDTO dto) {
        return deviceMapper.batchUpdateModel(dto);
    }

    @Override
    public int updateServiceRate(DeviceVO data) {
        return deviceMapper.updateServiceRate(data);
    }

    @Override
    public int renewalRentTime(Long deviceId, int time, TimeUnit timeUnit) {
        long seconds = timeUnit.toSeconds(time);
        return deviceMapper.renewalRentTime(deviceId, seconds);
    }

    @Override
    public void pullDeviceInfo(Long deviceId, String onlineType) {
        DeviceVO device = selectById(deviceId);
        pullDeviceInfo(device, onlineType);
    }

    @Override
    public boolean addEle(Long deviceId, BigDecimal ele, String reason, boolean requiredIot) {
        DeviceVO device = selectById(deviceId);
        return this.addEle(device, ele, reason, requiredIot);
    }

    @Override
    public boolean addEle(DeviceVO device, BigDecimal amount, String reason, boolean requiredIot) {
        ServiceUtil.assertion(device == null || device.getDeviceId() == null, "设备不存在");
        ServiceUtil.assertion(amount == null, "增加的电量不能为空");
        ServiceUtil.assertion( amount.compareTo(BigDecimal.ZERO) < 0, "增加的电量不允许小于0");
        ServiceUtil.assertion(StringUtils.isAllBlank(device.getMac(), device.getMac2()), "设备MAC号为空");

        Integer result = transactionTemplate.execute(status -> {
            // 更新设备电量
            int update = this.addEleDb(device.getDeviceId(), amount);
            ServiceUtil.assertion(update != 1, "更新电量失败");

            // 操作物联网设备
            try {
                CommandResponse res = this.syncEle(device.getDeviceId(), reason);
                ServiceUtil.assertion(res == null, "电量充值失败：未知错误");
                ServiceUtil.assertion(!res.isSuccess(), "为设备充值电量失败:%s", res.getMsg());
            } catch (Exception e) {
                if (requiredIot) {
                    throw e;
                }
            }

            return update;
        });

        return result != null && result == 1;
    }

    @Override
    public int updateLastRecoverTime(Long deviceId, Long lastRecoverTime) {
        Device data = new Device();
        data.setDeviceId(deviceId);
        data.setLastRecoverTime(lastRecoverTime);
        return deviceMapper.updateSmDevice(data);
    }

    @Override
    public int setTime(DeviceVO device, LocalDateTime expireTime, int tryCount, String reason) {
        if (device == null || StringUtils.isBlank(device.getMac()) || expireTime == null) {
            return 0;
        }

        Integer result = transactionTemplate.execute(status -> {
            // 更新数据库
            int update = this.updateExpireTime(device.getDeviceId(), expireTime);
            ServiceUtil.assertion(update != 1, "更新数据失败");

            // 操作设备
            Duration between = Duration.between(LocalDateTime.now(), expireTime);
            long seconds = between.getSeconds();
            if (seconds > 0) {
                CommandResponse res = iotService.trySetTime(device, seconds, tryCount, reason);
                if (!res.isSuccess()) {
                    log.error("操作设备设置时长{}失败：{}", device.getDeviceId(), res.getMsg());
                    throw new ServiceException(res.getMsg());
                }
            }

            return update;
        });

        return result == null ? 0 : result;
    }

    private int updateExpireTime(Long deviceId, LocalDateTime expireTime) {
        Device data = new Device();
        data.setDeviceId(deviceId);
        data.setExpireTime(expireTime);
        return deviceMapper.updateSmDevice(data);
    }

    @Override
    public int setWifi(DeviceWifiDTO dto) {
        DeviceVO device = selectById(dto.getDeviceId());
        if (device == null) {
            return 0;
        }
        CommandResponse res = iotService.setWifi(device, dto.getWifiName(), dto.getWifiPwd(), dto.getReason());
        ServiceUtil.assertion(!res.isSuccess(), "设置WIFI失败：" + res.getMsg());

        return 1;
    }

    @Override
    public DeviceVO selectByAnyMac(String mac) {
        return deviceMapper.selectByAnyMac(mac);
    }

    @Override
    public void pullDeviceInfo(DeviceVO device, String onlineType) {
        this.pullDeviceInfoList(Collections.singletonList(device), onlineType);
    }

    @Override
    public int unbindAgent(Long deviceId) {
        DeviceVO device = selectById(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        ServiceUtil.assertion(device.getAgentId() == null, "设备没有代理商，无需解绑");

        Integer result = transactionTemplate.execute(status -> {
            // 解绑
            int unbind = deviceMapper.unbindAgent(deviceId);
            ServiceUtil.assertion(unbind != 1, "解绑失败，请刷新后重试");

            // 新增解绑记录
            smDeviceBindRecordService.record(device.getAgentId(), deviceId, BindRecordType.UNBIND, BindRecordUserType.AGENT);

            return unbind;
        });

        return result == null ? 0 : result;
    }

    @Override
    public String bindMch(Long deviceId, Long mchId) {
        DeviceVO device = this.selectById(deviceId);
        return this.bind(null, mchId, device);
    }

    @Override
    public BigDecimal selectMaxAgentServiceRate(DeviceQuery query) {
        query.setServiceMode(DeviceServiceMode.AGENT.getMode());
        return deviceMapper.selectMaxAgentServiceRate(query);
    }

    @Override
    public int bindAgent(Long deviceId, Long agentId, BigDecimal agentServiceRate) {
        if (deviceId == null || agentId == null || agentServiceRate == null) {
            return 0;
        }
        ServiceUtil.assertion(!userValidator.isAgent(agentId), "该用户不是代理商，无法绑定为设备代理商");

        Integer result = transactionTemplate.execute(status -> {
            // 绑定
            int bind = deviceMapper.bindAgent(deviceId, agentId, agentServiceRate);
            ServiceUtil.assertion(bind != 1, "绑定代理商失败，请刷新后重试");

            // 后校验
            DeviceVO vo = this.selectById(deviceId);
            deviceValidator.checkAgent(vo);
            deviceAssembler.assembleBonusList(vo);
            deviceValidator.checkBonus(vo);

            // 添加绑定记录
            int record = smDeviceBindRecordService.record(agentId, deviceId, BindRecordType.BIND, BindRecordUserType.AGENT);
            ServiceUtil.assertion(record != 1, "添加绑定记录失败");

            return bind;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int setEle(DeviceVO device, BigDecimal ele, int tryCount, String reason) {
        if (device == null || ele == null || tryCount <= 0) {
            log.error("设置电量失败：参数为空");
            return 0;
        }
        Integer result = transactionTemplate.execute(status -> {
            int update = this.updateSurplusEle(device.getDeviceId(), ele);
            ServiceUtil.assertion(update != 1, "更新设备数据失败");

            CommandResponse res = iotService.trySetEle(device, ele, tryCount, reason);
            if (res == null || !res.isSuccess()) {
                log.error("操作设备设置电量{}失败：{}", device.getDeviceId(), res);
                ServiceUtil.assertion(res == null, "充值电量失败");
                IotHttpStatus statusResult = IotHttpStatus.convertByCode(res.getCode());
                ServiceUtil.assertion(!res.isSuccess(), "充值电量失败：" + statusResult.getMsg());
            }

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int setTime(DeviceVO device, long seconds, int tryCount, String reason) {
        return this.setTime(device, LocalDateTime.now().plusSeconds(seconds), tryCount, reason);
    }

    @Override
    public List<DeviceVO> selectStaffDeviceList(DeviceQuery query) {
        query.setInStaff(true);
        return deviceMapper.selectSmDeviceList(query);
    }

    private int updateSurplusEle(Long deviceId, BigDecimal ele) {
        if (deviceId == null || ele == null) {
            return 0;
        }
        Device data = new Device();
        data.setSurplusEle(ele);
        data.setDeviceId(deviceId);
        return deviceMapper.updateSmDevice(data);
    }


    @Override
    public boolean addTime(Long deviceId, long seconds, String reason, boolean requiredIot) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        return addTime(device, seconds, reason, requiredIot);
    }

    @Override
    public boolean addTime(DeviceVO device, long seconds, String reason, boolean requiredIot) {
        ServiceUtil.assertion( seconds < 0, "增加的时长不允许小于0");
        ServiceUtil.assertion(device == null || device.getDeviceId() == null, "设备不存在");
        ServiceUtil.assertion(StringUtils.isAllBlank(device.getMac(), device.getMac2()), "设备MAC号为空");

        Long deviceId = device.getDeviceId();
        LocalDateTime now = LocalDateTime.now();

        Integer result = transactionTemplate.execute(status -> {
            // 更新数据库时长
            int updateCount = this.addTimeDb(deviceId, seconds);
            ServiceUtil.assertion(updateCount != 1, "增加时长失败，请刷新后重试");

            // 同步物联网设备时长
            try {
                CommandResponse res = this.syncTime(deviceId, now, reason);
                ServiceUtil.assertion(res == null, "设备充值失败：未知错误");
                ServiceUtil.assertion(!res.isSuccess(), "设备充值失败:%s", res.getMsg());
            } catch (Exception e) {
                if (requiredIot) {
                    throw e;
                }
            }

            return updateCount;
        });

        // 成功后拉取设备信息
        boolean success = result != null && result == 1;
        if (success) {
            this.pullDeviceInfoAsync(device, 3, TimeUnit.SECONDS);
        }

        return success;
    }

    @Override
    public void freshStatus(DeviceVO device) {
        if (device == null || device.getExpireTime() == null) {
            return;
        }
        if (!DeviceStatus.USING.getStatus().equals(device.getStatus())) {
            return;
        }
        LocalDateTime now = LocalDateTime.now();
        Duration between = Duration.between(now, device.getExpireTime());

        // 如果已过期，则调整设备为正常状态
        int i = deviceMapper.changeTimeoutStatus(device.getDeviceId());
        if (i == 1) {
            return;
        }

//        // 如果没有过期，则延迟至过期时间继续查询
//        if (between.getSeconds() > 0) {
//            scheduledExecutorService.schedule(() -> {
//                freshStatus(device.getDeviceId());
//            }, between.getSeconds() , TimeUnit.SECONDS);
//        }
    }

    /**
     * 设备心跳，用于检测是否在线
     */
    @Override
    public void deviceHeartBeat() {
        log.info("device heart beat running...");
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(new DeviceQuery());
        if (CollectionUtils.isEmpty(list)) {
            log.info("device list is empty");
            return;
        }
        for (DeviceVO device : list) {
            this.setOnlineStatus(device, IotConstants.ONLINE_TYPE_COMMAND);
        }
        int i = deviceMapper.batchUpdateOnlineStatus(list);
        log.info("update count: {}", i);
    }

    @Override
    public int logicDel(List<Long> deviceIds) {
        // 校验
        this.validatePreLogicDelete(deviceIds);

        // 查询设备列表
        List<DeviceVO> deviceList = this.selectByIds(deviceIds);
        // 拉取最新数据
        this.pullDeviceInfoList(deviceList, null);

        Integer result = transactionTemplate.execute(status -> {
            // 关闭设备订单
            for (Long deviceId : deviceIds) {
                if (deviceId == null) {
                    continue;
                }

                DeviceVO device = deviceList.stream().filter(item -> Objects.equals(item.getDeviceId(), deviceId)).findFirst().orElse(null);
                if (device == null) {
                    continue;
                }

                // 关闭设备未结束的订单
                this.closeNotFinishedBill(device, device.getTotalElectriQuantity());
            }

            int i = deviceMapper.logicDel(deviceIds);
            ServiceUtil.assertion(i != deviceIds.size(), "删除设备失败,请刷新后重试");

            return i;
        });

        return result == null ? 0 : result;
    }

    @Override
    public Map<Long, List<DeviceVO>> selectGroupByModelIds(List<Long> modelIds) {
        if (CollectionUtils.isEmpty(modelIds)) {
            return Collections.emptyMap();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setModelIds(modelIds);
        List<DeviceVO> list = deviceMapper.selectSmDeviceList(dto);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.groupingBy(DeviceVO::getModelId));
    }

    /**
     * 切换设备通电状态（通/断电）
     *
     * @param device       设备id
     * @param powerStatus  通电状态
     * @param reason
     * @param openCommand
     * @param closeCommand
     */
    @Override
    public int switchDevice(DeviceVO device, DevicePowerStatus powerStatus, String reason, String openCommand, String closeCommand) {
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");

        Integer result = transactionTemplate.execute(status -> {
            // 修改数据库
            Device data = new Device();
            data.setDeviceId(device.getDeviceId());
            data.setPowerStatus(powerStatus.getStatus());
            data.setUserOperaTime(LocalDateTime.now());
            data.setUserOperaEle(device.getTotalElectriQuantity());
            if (DevicePowerStatus.ON.equals(powerStatus)) {
                data.setUserOperaType(DeviceUserOperaType.OPEN.getType());
            } else {
                data.setUserOperaType(DeviceUserOperaType.CLOSE.getType());
            }
            int update = deviceMapper.updateSmDevice(data);

            // 操作设备
            if (update == 1) {
                if (DevicePowerStatus.ON.equals(powerStatus)) {
                    boolean open = iotService.open(device, reason, openCommand);
                    ServiceUtil.assertion(!open, "开启设备失败");
                } else if (DevicePowerStatus.OFF.equals(powerStatus)) {
                    boolean close = iotService.close(device, reason, closeCommand);
                    ServiceUtil.assertion(!close, "关闭设备失败");
                } else {
                    throw new ServiceException("不支持的操作");
                }
            }

            return update;
        });


        return result == null ? 0 : result;
    }

    @Override
    public void pullDeviceInfoList(List<DeviceVO> list, String onlineType) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 拼接正在使用中的订单数量
        deviceAssembler.assembleUsingBillCount(list);

        // 列表根据产品ID分组
        Map<String, List<DeviceVO>> group = list.stream()
                .filter(item -> StringUtils.hasText(item.getModelProductId()))
                .collect(Collectors.groupingBy(DeviceVO::getModelProductId));

        for (Map.Entry<String, List<DeviceVO>> entry : group.entrySet()) {

            // 批量获取设备信息
            List<DeviceVO> groupDeviceList = entry.getValue();
            String productId = entry.getKey();
            List<IotDeviceInfo> iotList = iotService.getDeviceInfo(groupDeviceList, productId);

            // 构造更新设备信息
            for (DeviceVO device : groupDeviceList) {
                // 获取数据点信息
                IotDeviceInfo deviceInfo = iotList.stream()
                        .filter(item -> Objects.equals(device.getMac(), item.getMac()) || Objects.equals(device.getMac2(), item.getMac()))
                        .findFirst().orElse(null);

                // 更新设备信息
                this.setDeviceInfo(device, deviceInfo);

                // 判断设备是否正在使用
                // 若开关开启，则正在使用
                boolean hasOpen = DevicePowerStatus.ON.getStatus().equals(device.getPowerStatus());
                // 若设备有正在使用中的订单，则正在使用
                boolean hasUsingBill = device.getUsingBillCount() != null && device.getUsingBillCount() > 0;

                // 有订单为使用中，无订单且通电为调度中，否则为空闲
                if (hasUsingBill) {
                    device.setStatus(DeviceStatus.USING.getStatus());
                } else if (hasOpen) {
                    device.setStatus(DeviceStatus.ADMIN.getStatus());
                } else {
                    device.setStatus(DeviceStatus.NORMAL.getStatus());
                }

            }
        }

        // 异步更新设备信息
        for (DeviceVO device : list) {
            scheduledExecutorService.schedule(() -> {
                if (StringUtils.hasText(onlineType)) {
                    // 获取并设置在线状态
                    this.setOnlineStatus(device, onlineType);
                }
                // 更新数据库
                this.updateIotInfo(device);
            }, 0, TimeUnit.SECONDS);
        }
    }

    private void setDeviceInfo(DeviceVO device, IotDeviceInfo iot) {
        if (device == null || iot == null) {
            return;
        }
        device.setLastPullTime(iot.getAt());
        device.setPowerStatus(iot.getS());
        device.setRemainTime(iot.getTime());
        device.setRealTimePower(iot.getP());
        device.setVoltage(iot.getV());
        device.setElectricity(iot.getA());
        device.setVersion(iot.getVersion());
        device.setVxs(iot.getVxs());
        device.setImsi(iot.getImsi());
        device.setSet(iot.getSet());
        device.setWxs(iot.getWxs());

        // 判断是否支持WIFI，支持则更新
        if (ModelTag.hasTag(device.getModelTags(), ModelTag.WIFI)) {
            device.setWifi(iot.getWifi());
        }

        // 判断是否支持电量，支持则更新
        if (ModelTag.hasTag(device.getModelTags(), ModelTag.ELE)) {
            device.setSurplusEle(iot.getM());
        } else {
            device.setSurplusEle(BigDecimal.ZERO);
        }

        // 总用电量
        if (iot.getW() != null) {
            // 若有数据点中的值大于数据库中的值，则更新
            if (iot.getW().compareTo(device.getTotalElectriQuantity()) >= 0) {
                device.setTotalElectriQuantity(iot.getW());
            }
            // 若数据点小于数据库中的值，则发命令恢复物联网设备的总用电量
            else {
                scheduledExecutorService.schedule(() -> {
                    String reason = "拉取设备信息时，恢复总用电量,数据库值:" + device.getTotalElectriQuantity() + "，OneNet值：" + iot.getW();
                    iotService.setTotalEle(device, device.getTotalElectriQuantity().multiply(new BigDecimal(1000)), reason);
                }, 0, TimeUnit.SECONDS);
            }
        }
    }

    private void setOnlineStatus(DeviceVO device, String onlineType) {
        // 是否在线
        String onlineStatus1 = iotService.getOnlineStatus(device.getMac(), device.getProductId(), onlineType);
        String onlineStatus2 = iotService.getOnlineStatus(device.getMac2(), device.getProductId(), onlineType);

        // 若有一个没获取到锁，则不更新状态
        boolean unlockMac1 = StringUtils.isNotBlank(device.getMac()) && onlineStatus1 == null;
        boolean unlockMac2 = StringUtils.isNotBlank(device.getMac2()) && onlineStatus2 == null;
        if (unlockMac1 || unlockMac2){
            device.setOnlineStatus(null);
            device.setOnlineStatus1(null);
            device.setOnlineStatus2(null);
            device.setLastOnlineTime(null);
        } else {
            device.setOnlineStatus1(onlineStatus1);
            device.setOnlineStatus2(onlineStatus2);
            if (DeviceOnlineStatus.ONLINE.getStatus().equals(onlineStatus1) || DeviceOnlineStatus.ONLINE.getStatus().equals(onlineStatus2)) {
                device.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
                device.setLastOnlineTime(LocalDateTime.now());
            } else {
                device.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
            }
        }
    }

    /**
     * 更新物联网信息
     */
    private int updateIotInfo(Device device) {
        Device data = new Device();
        data.setDeviceId(device.getDeviceId());
        data.setLastPullTime(device.getLastPullTime());
        data.setTotalElectriQuantity(device.getTotalElectriQuantity());
        data.setPowerStatus(device.getPowerStatus());
        data.setRemainTime(device.getRemainTime());
        data.setRealTimePower(device.getRealTimePower());
        data.setVoltage(device.getVoltage());
        data.setElectricity(device.getElectricity());
        data.setWifi(device.getWifi());
        data.setSurplusEle(device.getSurplusEle());
        data.setStatus(device.getStatus());
        data.setOnlineStatus(device.getOnlineStatus());
        data.setOnlineStatus1(device.getOnlineStatus1());
        data.setOnlineStatus2(device.getOnlineStatus2());
        data.setLastOnlineTime(device.getLastOnlineTime());
        data.setVersion(device.getVersion());
        data.setImsi(device.getImsi());
        data.setSet(device.getSet());
        data.setWxs(device.getWxs());
        return deviceMapper.updateSmDevice(data);
    }

    /**
     * 异步延迟执行拉取设备信息
     * @param device 设备id列表
     * @param delay 延迟时间
     * @param unit 时间单位
     */
    private void pullDeviceInfoAsync(DeviceVO device, long delay, TimeUnit unit) {
        scheduledExecutorService.schedule(() -> {
            log.info("异步拉取设备信息 start");
            this.pullDeviceInfo(device, null);
            log.info("异步拉取设备信息 end");
        }, delay, unit);
    }



    @Override
    public String bindBySn(Long storeId, Long userId, String deviceNo) {
        DeviceVO device = selectByDeviceNo(deviceNo);
        return this.bind(storeId, userId, device);
    }

    @Override
    public String bindByMac(Long storeId, Long userId, String mac) {
        DeviceVO device = selectByMac(mac);
        return this.bind(storeId, userId, device);
    }

    private String bind(Long storeId, Long userId, DeviceVO device) {
        ServiceUtil.assertion(device == null, "设备未录入");
        // 如果绑定的用户是原来的用户，则直接返回MAC
        if (device.getUserId() != null) {
            if (Objects.equals(device.getUserId(), userId)) {
                return device.getMac();
            } else {
                throw new ServiceException("该设备已被绑定");
            }
        }
        // 如果绑定的是原来的店铺，则直接返回MAC
        if (device.getStoreId() != null) {
            if (Objects.equals(device.getStoreId(), storeId)) {
                return device.getMac();
            } else {
                throw new ServiceException("该设备已被绑定");
            }
        }

        ServiceUtil.assertion(device.getLimitRechargeTime() != null
                && device.getLimitRechargeTime().isAfter(LocalDateTime.now()),
                "设备无法绑定：" + device.getLimitRechargeReason()
        );

        // 查询用户
        SmUserVO user = userService.selectSimpleById(userId);
        ServiceUtil.assertion(user == null, "用户不存在");

        // 查询店铺
        if (storeId != null) {
            StoreVo store = storeService.selectSmStoreById(storeId);
            ServiceUtil.assertion(store == null, "店铺不存在");
            ServiceUtil.assertion(!Objects.equals(store.getUserId(), userId), "该店铺不属于该用户");
        }

        transactionTemplate.execute(status -> {
            // 绑定
            int updateCount = deviceMapper.bind(device.getDeviceId(), storeId, userId);
            ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

            // 校验分成
            deviceValidator.checkBonus(device.getDeviceId());

            // 记录绑定记录
            int record = smDeviceBindRecordService.record(userId, device.getDeviceId(), BindRecordType.BIND, BindRecordUserType.MCH);
            ServiceUtil.assertion(record != 1, "添加绑定记录失败");

            // 用户设置为商户
            if (UserType.USER.getType().equals(user.getType())) {
                int changeType = userService.changeType(userId, UserType.MCH);
                ServiceUtil.assertion(changeType != 1, "修改用户类型失败");
            }
            return updateCount;
        });

        return device.getMac();
    }

    /**
     * 根据mac号查询数据
     * @param mac
     * @return
     */
    @Override
    public DeviceVO selectByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return null;
        }
        return deviceMapper.selectSimpleSmDeviceByMac(mac);
    }

    @Override
    public int resetWithBill(Long deviceId, boolean requiredIot, BigDecimal totalEle, String reason, boolean withTime, boolean withEle, String timeCommand) {
        ServiceUtil.assertion(deviceId == null, "设备id不能为空");

        // 获取设备信息
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        // 拉取最新设备信息
        pullDeviceInfo(device, null);

        // 关闭未完成订单
        this.closeNotFinishedBill(device, totalEle);

        Integer result = transactionTemplate.execute(status -> {
            // 修改数据库设备信息
            Device data = new Device();
            data.setDeviceId(deviceId);
            data.setStatus(DeviceStatus.NORMAL.getStatus());
            if (withTime) {
                data.setExpireTime(LocalDateTime.now());
            }
            if (withEle) {
                data.setSurplusEle(BigDecimal.ZERO);
                data.setExpireEle(device.getTotalElectriQuantity());
            }
            data.setUserOperaType(DeviceUserOperaType.RESET.getType());
            data.setUserOperaEle(device.getTotalElectriQuantity());
            data.setUserOperaTime(LocalDateTime.now());
            int update = deviceMapper.updateSmDevice(data);
            ServiceUtil.assertion(update != 1, "更新设备%s信息失败", device.getDeviceNo());

            // 发送命令
            if (withTime) {
                CommandResponse res = iotService.setTime(device, 0L, reason, timeCommand);
                if (requiredIot) {
                    ServiceUtil.assertion(res == null, "归零电量失败，返回值为空");
                    ServiceUtil.assertion(!res.isSuccess(), "归零电量失败，请检查设备是否在线或联系管理员");
                }
            }
            if (withTime && withEle) {
                try {
                    Thread.sleep(5000);
                } catch (Exception e) {
                    log.error("设备归零等待时间异常:{}", e.getMessage());

                    throw new ServiceException(e.getMessage());
                }
            }
            if (withEle) {
                CommandResponse res = iotService.setEle(device, BigDecimal.ZERO, reason);
                if (requiredIot) {
                    ServiceUtil.assertion(res == null, "归零电量失败，返回值为空");
                    ServiceUtil.assertion(!res.isSuccess(), "归零电量失败，请检查设备是否在线或联系管理员");
                }
            }

            return 1;
        });

//        if (result != null && result == 1) {
//            this.resetRecord(device, RecordTimeType.TIME, reason);
//            this.resetRecord(device, RecordTimeType.ELE, reason);
//        }

        return result == null ? 0 : result;
    }

    /**
     * 尝试关闭该设备未结束的所有订单
     * @param device 设备
     * @param totalEle 总用电量
     */
    @Override
    public int closeNotFinishedBill(DeviceVO device, BigDecimal totalEle) {
        if (device == null || device.getDeviceId() == null) {
            return 0;
        }

        TransactionBillQuery query = new TransactionBillQuery();
        query.setStatusList(TransactionBillStatus.canClose());
        query.setDeviceId(device.getDeviceId());
        query.setIsFinished(false);   // 未结束的订单都会被关闭
        List<TransactionBillVO> billList = transactionBillService.selectSmTransactionBillList(query);
        if (CollectionUtils.isEmptyElement(billList)) {
            return 0;
        }
        transactionAssembler.assembleBonusList(billList);

        return transactionBillService.batchCloseBillByDevice(billList, device, totalEle);
    }

    @Override
    public int initTotalEle(Long deviceId) {
        if (deviceId == null) {
            return 0;
        }
        // 拉取设备信息
        this.pullDeviceInfo(deviceId, null);
        return deviceMapper.initTotalEle(deviceId);
    }

    @Override
    public int updateVxs(Long deviceId, BigDecimal vxs, String reason) {
        if (deviceId == null || vxs == null) {
            return 0;
        }
        DeviceVO device = this.selectById(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");

        CommandResponse res = iotService.setVxs(device, vxs, reason);
        return res != null && res.isSuccess() ? 1 : 0;
    }

    @Override
    public int uploadData(DeviceVO device, String reason) {
        ServiceUtil.assertion(device == null, "设备不存在");
        CommandResponse res = iotService.uploadData(device, reason, true);
        return res != null && res.isSuccess() ? 1 : 0;
    }

    @Override
    public int reboot(DeviceVO device, String reason) {
        ServiceUtil.assertion(device == null, "设备不存在");
        CommandResponse res = iotService.reboot(device, reason);
        return res != null && res.isSuccess() ? 1 : 0;
    }

    /**
     * 增加数据库设备时长
     *
     * @param deviceId 设备ID
     * @param seconds  秒
     * @return
     */
    @Override
    public int addTimeDb(Long deviceId, long seconds) {
        if (deviceId == null || seconds <= 0) {
            return 0;
        }
        return deviceMapper.addTime(deviceId, seconds);
    }

    @Override
    public int addEleDb(Long deviceId, BigDecimal amount) {
        if (deviceId == null || amount == null || amount.compareTo(BigDecimal.ZERO) <= 0) {
            return 0;
        }
        return deviceMapper.addEle(deviceId, amount);
    }

    @Override
    public CommandResponse syncTime(Long deviceId, LocalDateTime now, String reason, String command) {
        if (deviceId == null) {
            return null;
        }
        DeviceVO device = selectById(deviceId);
        if (device == null) {
            return null;
        }

        long syncSeconds = DeviceUtils.getSyncSeconds(device.getExpireTime(), now);
        if (syncSeconds > 0) {
            return iotService.setTime(device, syncSeconds, reason, command);
        }
        return null;
    }

    @Override
    public CommandResponse syncEle(Long deviceId, String reason) {
        if (deviceId == null) {
            return null;
        }
        DeviceVO device = selectById(deviceId);

        return iotService.setEle(device, device.getSurplusEleDb(), reason);
    }

    /**
     * 监控设备信息，并对设备进行校准
     */
    @Override
    public void monitor(List<Long> deviceIds) {
        // 若不为空，则查询所有设备
        if (CollectionUtils.isEmpty(deviceIds)) {
            DeviceQuery query = new DeviceQuery();
            query.setDeviceIds(deviceIds);
            deviceIds = this.selectIds(query);
        }
        if (CollectionUtils.isEmptyElement(deviceIds)) {
            return;
        }

        for (Long deviceId : deviceIds) {
            try {
                scheduledExecutorService.schedule(() -> {
                    // 获取设备最新信息
                    DeviceVO device = selectById(deviceId);
                    pullDeviceInfo(device, null);

                    // 若物联网设备的时长或者电量小于当前设备数据库的值，则执行一次同步（开启）
                    // 一般情况下，物联网设备的值会大于或等于当前数据库的值
                    // 恢复时长：数据库时长大于0时，且当前设备时长小于数据库时长
                    long deviceSeconds = device.getRemainTime() == null ? 0 : device.getRemainTime().longValue();
                    if (device.getSurplusSecondsDb() > 0 && device.getSurplusSecondsDb() > deviceSeconds) {
                        this.syncTime(device.getDeviceId(), LocalDateTime.now(), "设备监控-同步时长");
                    }

                    // 恢复电量：数据库电量大于0时，且当前设备电量小于数据库电量
                    BigDecimal surplusEleDb = device.getSurplusEleDb();
                    if (surplusEleDb.compareTo(BigDecimal.ZERO) > 0 && surplusEleDb.compareTo(device.getSurplusEle()) > 0  ) {
                        this.syncEle(device.getDeviceId(), "设备监控-同步电量");
                    }

                    // 根据上一次用户的操作来判断该怎么关闭设备
                    // 若上次操作为充值时间，则判断是否还有剩余时间
                    boolean isClose = DevicePowerStatus.OFF.getStatus().equals(device.getPowerStatus());   // 是否关闭
                    if (DeviceUserOperaType.TIME.getType().equals(device.getUserOperaType())) {
                        boolean hasTime = device.getSurplusSecondsDb() > 0;         // 剩余时间
                        if (!isClose && !hasTime) {
                            this.switchDevice(device, DevicePowerStatus.OFF, "设备监控-发现异常未关闭设备(时长)");
                        }
                    }
                    // 若上次操作为充值电量，则判断是否还有剩余电量
                    else if (DeviceUserOperaType.ELE.getType().equals(device.getUserOperaType())) {
                        boolean hasEle = device.getSurplusEleDb().compareTo(BigDecimal.ZERO) > 0;   // 剩余电量
                        if (!isClose && !hasEle) {
                            this.switchDevice(device, DevicePowerStatus.OFF, "设备监控-发现异常未关闭设备(电量)");
                        }
                    }
                    // 若上次操作为归零，则判断是否关闭
                    else if (DeviceUserOperaType.RESET.getType().equals(device.getUserOperaType())) {
                        if (!isClose) {
                            this.switchDevice(device, DevicePowerStatus.OFF, "设备监控-发现异常未关闭设备(归零)");
                        }
                    }

                },0, TimeUnit.SECONDS);

            } catch (Exception e) {
                log.error("监控设备id={}出错：{}", deviceId, e.getMessage());
            }
        }
    }

    @Override
    public int bltSyncIot(IotDeviceInfo info) {
        if (info == null) {
            return 0;
        }

        // 查询设备
        DeviceVO device = selectByAnyMac(info.getMac());
        ServiceUtil.assertion(device == null, "设备不存在");

        // 更新数据库总用电量
        this.setDeviceInfo(device, info);

        return this.updateIotInfo(device);
    }

    @Override
    public List<DeviceVO> selectByMchId(Long userId) {
        if (userId == null) {
            return Collections.emptyList();
        }
        DeviceQuery query = new DeviceQuery();
        query.setUserId(userId);
        return selectSmDeviceList(query);
    }

    @Override
    public int updateOnlineStatusByMac(String mac, String status) {
        if (StringUtils.isAnyBlank(mac, status)) {
            return 0;
        }
        return deviceMapper.updateOnlineStatusByMac(mac, status);
    }

    private List<Long> selectIds(DeviceQuery query) {
        return deviceMapper.selectIds(query);
    }

    /**
     * 设备是否已经被绑定
     *
     * @param mac
     * @return
     */
    @Override
    public int bindCodeByMac(String mac) {
        if (StringUtils.isBlank(mac)) {
            return Constants.DEVICE_NOT_FOUND;
        }
        DeviceVO device = deviceMapper.selectSimpleSmDeviceByMac(mac);
        return isBindCode(device);
    }

    private int isBindCode(DeviceVO device) {
        if (device == null) {
            return Constants.DEVICE_NOT_FOUND;
        }
        if (device.getUserId() == null) {
            return Constants.DEVICE_NOT_BIND;
        } else {
            return Constants.DEVICE_BINDED;
        }
    }

    /**
     * 根据SN码查询设备是否已经绑定
     *
     * @param deviceNo
     * @return
     */
    @Override
    public int bindCodeBySn(String deviceNo) {
        if (StringUtils.isBlank(deviceNo)) {
            return Constants.DEVICE_NOT_FOUND;
        }
        DeviceVO device = deviceMapper.selectSimpleSmDeviceBySn(deviceNo);
        return isBindCode(device);
    }

    /**
     * 更新设备服务费
     *
     * @param userId
     * @param serviceRate
     */
    @Override
    public int updateServiceRateByUserId(Long userId, BigDecimal serviceRate) {
        if (userId == null) {
            return 0;
        }
        List<StoreVo> list = storeService.selectByUserId(userId);
        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }
        return deviceMapper.updateServiceRateByStores(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()), serviceRate);
    }

    @Override
    public int updateByIot(Device device) {
        device.setLastPullTime(DateUtils.getNowDate());
        return deviceMapper.updateSmDeviceByIot(device);
    }

    /**
     * ids查询列表
     *
     * @param ids
     * @param userId
     */
    @Override
    public List<DeviceVO> selectByIds(List<Long> ids, Long userId) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        DeviceQuery dto = new DeviceQuery();
        dto.setDeviceIds(ids);
        dto.setUserId(userId);
        return selectSmDeviceList(dto);
    }

    /**
     * 查询各店铺的设备数量,并店铺
     *
     * @param storeIds 店铺id
     */
    @Override
    public Map<Long, Integer> selectCountMapByStoreIds(List<Long> storeIds) {
        if (CollectionUtils.isEmptyElement(storeIds)) {
            return Collections.emptyMap();
        }
        List<DeviceCountVO> list = deviceMapper.selectCountByStoreIds(storeIds);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(DeviceCountVO::getStoreId, DeviceCountVO::getCount));
    }

    /**
     * 条件查询数量，并店铺
     *
     * @param query 查询条件
     * @param keyMapper 键值的映射
     */
    @Override
    public <K> Map<K, Integer> selectCommonCountMap(DeviceQuery query, Function<? super DeviceCountVO, ? extends K> keyMapper ) {
        if (query == null) {
            return Collections.emptyMap();
        }
        List<DeviceCountVO> list = deviceMapper.selectCommonCount(query);
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, DeviceCountVO::getCount));
    }

    /**
     * 获取设备，包含套餐列表
     *
     * @param deviceSn
     * @return
     */
    @Override
    public DeviceVO getDeviceWithSuitList(String deviceSn) {
        if (deviceSn == null) {
            return null;
        }
        DeviceVO device = deviceMapper.selectByDeviceNo(deviceSn);
        if (device != null) {
            deviceAssembler.assembleSuitList(Collections.singletonList(device));
        }
        return device;
    }

    /**
     * 根据设备编号查询
     *
     * @param deviceNo
     */
    @Override
    public DeviceVO selectByDeviceNo(String deviceNo) {
        return deviceMapper.selectByDeviceNo(deviceNo);
    }

    @Override
    @Transactional
    public int unbind(Long deviceId) {
        DeviceVO device = deviceMapper.selectSmDeviceByDeviceId(deviceId);
        ServiceUtil.assertion(device == null || device.getDeleted(), "设备不存在");
        // 判断是否有正在使用中的订单
//        ServiceUtil.assertion(deviceValidator.hasUsingBill(device.getDeviceId()), "当前设备有正在使用中的订单，请关闭后重试");

        Integer result = transactionTemplate.execute(status -> {
            int updateCount = deviceMapper.unbindStore(deviceId);
            ServiceUtil.assertion(updateCount != 1, "当前设备信息已变更，请刷新后重试");

            // 删除套餐关联关系
            deviceSuitService.deleteByDeviceId(deviceId);

            // 新增解绑记录
            smDeviceBindRecordService.record(device.getUserId(), deviceId, BindRecordType.UNBIND, BindRecordUserType.MCH);

            return updateCount;
        });

        return result == null ? 0 : result;
    }

    /**
     * 设备绑定SN码，若其他设备已有该SN则直接替换
     *
     * @param deviceId 设备ID
     * @param sn       SN
     */
    @Override
    public int bindSn(Long deviceId, String sn) {
        if (deviceId == null || StringUtils.isBlank(sn)) {
            return 0;
        }
        return deviceMapper.bindSn(deviceId, sn);
    }

    @Override
    public boolean setWxs(Long deviceId, BigDecimal wxs, String reason) {
        if (deviceId == null || wxs == null) {
            return false;
        }
        DeviceVO device = selectById(deviceId);
        ServiceUtil.assertion(device == null, "ID为" + deviceId + "的设备不存在");
        return setWxs(device, wxs, reason);
    }

    public boolean setWxs(DeviceVO device, BigDecimal wxs, String reason) {
        if (device == null || wxs == null) {
            return false;
        }
        CommandResponse res = iotService.setWxs(device, wxs, reason);
        if (res == null || !res.isSuccess()) {
            return false;
        }
        return true;
    }

    @Override
    public boolean setSet(Long deviceId, String set, String reason) {
        if (deviceId == null || StringUtils.isBlank(set)) {
            return false;
        }
        DeviceVO device = selectById(deviceId);
        ServiceUtil.assertion(device == null, "ID为" + deviceId + "的设备不存在");
        return setSet(device, set, reason);
    }

    public boolean setSet(DeviceVO device, String set, String reason) {
        if (device == null || StringUtils.isBlank(set)) {
            return false;
        }
        CommandResponse res = iotService.setSet(device, set, reason);
        if (res == null || !res.isSuccess()) {
            return false;
        }
        return true;
    }
}
