package com.ruoyi.ss.hardVersion.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.hardVersion.domain.HardVersion;
import com.ruoyi.ss.hardVersion.domain.HardVersionQuery;
import com.ruoyi.ss.hardVersion.domain.HardVersionVO;
import com.ruoyi.ss.hardVersion.mapper.HardVersionMapper;
import com.ruoyi.ss.hardVersion.service.HardVersionService;

/**
 * 硬件版本Service业务层处理
 *
 * @author ruoyi
 * @date 2025-03-11
 */
@Service
public class HardVersionServiceImpl implements HardVersionService
{
    @Autowired
    private HardVersionMapper hardVersionMapper;

    /**
     * 查询硬件版本
     *
     * @param id 硬件版本主键
     * @return 硬件版本
     */
    @Override
    public HardVersionVO selectHardVersionById(Long id)
    {
        return hardVersionMapper.selectHardVersionById(id);
    }

    /**
     * 查询硬件版本列表
     *
     * @param hardVersion 硬件版本
     * @return 硬件版本
     */
    @Override
    public List<HardVersionVO> selectHardVersionList(HardVersionQuery hardVersion)
    {
        return hardVersionMapper.selectHardVersionList(hardVersion);
    }

    /**
     * 新增硬件版本
     *
     * @param hardVersion 硬件版本
     * @return 结果
     */
    @Override
    public int insertHardVersion(HardVersion hardVersion)
    {
        hardVersion.setCreateTime(DateUtils.getNowDate());
        return hardVersionMapper.insertHardVersion(hardVersion);
    }

    /**
     * 修改硬件版本
     *
     * @param hardVersion 硬件版本
     * @return 结果
     */
    @Override
    public int updateHardVersion(HardVersion hardVersion)
    {
        return hardVersionMapper.updateHardVersion(hardVersion);
    }

    /**
     * 批量删除硬件版本
     *
     * @param ids 需要删除的硬件版本主键
     * @return 结果
     */
    @Override
    public int deleteHardVersionByIds(Long[] ids)
    {
        return hardVersionMapper.deleteHardVersionByIds(ids);
    }

    /**
     * 删除硬件版本信息
     *
     * @param id 硬件版本主键
     * @return 结果
     */
    @Override
    public int deleteHardVersionById(Long id)
    {
        return hardVersionMapper.deleteHardVersionById(id);
    }

    /**
     * 查询一个
     *
     * @param query 查询条件
     * @return 硬件版本
     */
    @Override   
    public HardVersionVO selectOne(HardVersionQuery query) {
        List<HardVersionVO> list = selectHardVersionList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }
}
