package com.ruoyi.ss.store.service.impl;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.common.domain.vo.LongDecimalVO;
import com.ruoyi.common.domain.vo.LongIntegerVO;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.dashboard.device.service.DeviceDashboardService;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.service.BonusConverter;
import com.ruoyi.ss.device.domain.DeviceCountVO;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.enums.DeviceGroupBy;
import com.ruoyi.ss.device.domain.enums.DeviceOnlineStatus;
import com.ruoyi.ss.device.domain.enums.DeviceStatus;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.payBill.domain.PayBillQuery;
import com.ruoyi.ss.payBill.domain.enums.PayBillBstType;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.refund.domain.RefundQuery;
import com.ruoyi.ss.refund.domain.enums.RefundStatus;
import com.ruoyi.ss.refund.service.RefundService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.domain.vo.StoreOperatorVO;
import com.ruoyi.ss.store.service.StoreAssembler;
import com.ruoyi.ss.store.service.StoreValidator;
import com.ruoyi.ss.storeApply.domain.StoreApplyQuery;
import com.ruoyi.ss.storeApply.domain.StoreApplyVO;
import com.ruoyi.ss.storeApply.service.StoreApplyService;
import com.ruoyi.ss.storeStaff.domain.StoreStaffVO;
import com.ruoyi.ss.storeStaff.service.StoreStaffService;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;

/**
 * @author wjh
 *         2024/4/29
 */
@Service
public class StoreAssemblerImpl implements StoreAssembler {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private StoreStaffService storeStaffService;

    @Autowired
    private BonusConverter bonusConverter;

    @Autowired
    private RefundService refundService;

    @Autowired
    private PayBillService payBillService;

    @Autowired
    private StoreApplyService storeApplyService;

    @Autowired
    private DeviceDashboardService deviceDashboardService;

    /**
     * 拼接设备数量
     *
     * @param list 店铺列表
     */
    @Override
    public void assembleDeviceCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        Map<Long, Integer> countMap = deviceService
                .selectCountMapByStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setDeviceCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接营收数据
     *
     * @param list
     */
    @Override
    public void assembleRevenue(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 本日订单金额
        this.assembleTodayIncome(list);
        // 总订单金额
        this.assembleTotalIncome(list);
        // 本日充值订单量
        this.assembleTodayOrderCount(list);
        // 总订单量
        this.assembleTotalOrderCount(list);
        // 本月收入
        this.assembleMonthIncome(list);
        // 上月收入
        this.assembleLastMonthIncome(list);
        // 在线设备数量
        this.assembleOnlineCount(list);
        // 离线设备数量
        this.assembleOfflineCount(list);
        // 设备状态数量
        this.assembleDeviceStatus(list);
    }

    /**
     * 拼接设备状态数量
     *
     * @param list
     */
    public void assembleDeviceStatus(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 查询设备状态数量，并按店铺、状态分组
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        List<Map<String, Object>> countList = deviceDashboardService.selectCountGroupBy(query, Arrays.asList("sd.store_id", "sd.status"));

        for (StoreVo store : list) {
            Map<String, Object> countMap = countList.stream()
                .filter(item -> Objects.equals(item.get("store_id"), store.getStoreId()))
                .collect(Collectors.toMap(item -> item.get("status").toString(), item -> item.get("count")));
            if (countMap != null) {
                store.setDeviceStatusMap(countMap);
            }
        }
    }


    /**
     * 拼接本日订单量
     *
     * @param list
     */
    public void assembleTodayOrderCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        LocalDate now = LocalDate.now();

        // 查询订单量
        TransactionBillQuery rechargeQuery = new TransactionBillQuery();
        rechargeQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        rechargeQuery.setStatusList(TransactionBillStatus.payedOrder());
        rechargeQuery.setType(TransactionBillType.RECHARGE.getType());
        rechargeQuery.setCreateDate(now);
        Map<Long, Integer> countMap = transactionBillService.selectCountGroupByStoreId(rechargeQuery)
                .stream().collect(Collectors.toMap(LongIntegerVO::getKey, LongIntegerVO::getValue));

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setTodayOrderCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接总订单量
     *
     * @param list
     */
    public void assembleTotalOrderCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 查询订单量
        TransactionBillQuery rechargeQuery = new TransactionBillQuery();
        rechargeQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        rechargeQuery.setStatusList(TransactionBillStatus.payedOrder());
        rechargeQuery.setType(TransactionBillType.RECHARGE.getType());
        Map<Long, Integer> countMap = transactionBillService.selectCountGroupByStoreId(rechargeQuery)
                .stream().collect(Collectors.toMap(LongIntegerVO::getKey, LongIntegerVO::getValue));

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setTotalOrderCount(count == null ? 0 : count);
        }
    }


    /**
     * 拼接本月收入
     *
     * @param list
     */
    @Override
    public void assembleMonthIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 查询支付信息
        PayBillQuery query = new PayBillQuery();
        LocalDate now = LocalDate.now();
        query.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        query.setStatusList(PayBillStatus.payedList());
        query.setPayMonth(now.getMonthValue());
        query.setPayYear(now.getYear());
        Map<Long, BigDecimal> billMap = payBillService.selectSumOfAmountGroupByStoreId(query)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setMonth(now.getMonthValue());
        refundQuery.setYear(now.getYear());
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BigDecimal monthIncome = billMap.get(store.getStoreId());
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            if (monthIncome == null) {
                monthIncome = BigDecimal.ZERO;
            }
            if (refundAmount != null) {
                monthIncome = monthIncome.subtract(refundAmount);
            }
            store.setMonthIncome(monthIncome);
        }
    }

    /**
     * 拼接本日收入
     *
     * @param list
     */
    @Override
    public void assembleTodayIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        LocalDate now = LocalDate.now();

        // 查询支付信息
        PayBillQuery query = new PayBillQuery();
        query.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        query.setStatusList(PayBillStatus.payedList());
        query.setPayDate(now);
        Map<Long, BigDecimal> billMap = payBillService.selectSumOfAmountGroupByStoreId(query)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setCreateDate(now);
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BigDecimal todayIncome = billMap.get(store.getStoreId());
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            // 订单金额
            if (todayIncome == null) {
                todayIncome = BigDecimal.ZERO;
            }
            // 退款金额
            if (refundAmount == null) {
                refundAmount = BigDecimal.ZERO;
            }
            store.setTodayIncome(todayIncome.subtract(refundAmount));
        }
    }

    /**
     * 拼接总订单金额
     *
     * @param list
     */
    public void assembleTotalIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 查询支付信息
        PayBillQuery query = new PayBillQuery();
        query.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        query.setStatusList(PayBillStatus.payedList());
        Map<Long, BigDecimal> billMap = payBillService.selectSumOfAmountGroupByStoreId(query)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BigDecimal totalAmount = billMap.get(store.getStoreId());
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            // 订单金额
            if (totalAmount == null) {
                totalAmount = BigDecimal.ZERO;
            }
            // 退款金额
            if (refundAmount == null) {
                refundAmount = BigDecimal.ZERO;
            }
            store.setTotalIncome(totalAmount.subtract(refundAmount));
        }
    }

    /**
     * 拼接上月收入
     *
     * @param list
     */
    @Override
    public void assembleLastMonthIncome(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        // 上月
        LocalDate lastMonth = LocalDate.now().plusMonths(-1);

        // 查询支付信息
        PayBillQuery query = new PayBillQuery();
        query.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        query.setStatusList(PayBillStatus.payedList());
        query.setPayMonth(lastMonth.getMonthValue());
        query.setPayYear(lastMonth.getYear());
        Map<Long, BigDecimal> billMap = payBillService.selectSumOfAmountGroupByStoreId(query)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        // 查询退款信息
        RefundQuery refundQuery = new RefundQuery();
        refundQuery.setStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));
        refundQuery.setMonth(lastMonth.getMonthValue());
        refundQuery.setYear(lastMonth.getYear());
        refundQuery.setPayBillBstType(PayBillBstType.RECHARGE_ORDER.getType());
        refundQuery.setStatusList(RefundStatus.successList());
        Map<Long, BigDecimal> refundMap = refundService.selectSumOfRefundAmountGroupByStoreId(refundQuery)
                .stream().collect(Collectors.toMap(LongDecimalVO::getKey, LongDecimalVO::getValue));

        for (StoreVo store : list) {
            BigDecimal lastMonthIncome = billMap.get(store.getStoreId());
            BigDecimal refundAmount = refundMap.get(store.getStoreId());
            if (lastMonthIncome == null) {
                lastMonthIncome = BigDecimal.ZERO;
            }
            if (refundAmount != null) {
                lastMonthIncome = lastMonthIncome.subtract(refundAmount);
            }
            store.setLastMonthIncome(lastMonthIncome);
        }
    }

    /**
     * 拼接在线数量
     *
     * @param list
     */
    @Override
    public void assembleOnlineCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, DeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setOnlineCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接离线数量
     *
     * @param list
     */
    @Override
    public void assembleOfflineCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.OFFLINE.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, DeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setOfflineCount(count == null ? 0 : count);
        }
    }

    /**
     * 拼接可共享设备数量
     *
     * @param list
     */
    @Override
    public void assembleAvailableDeviceCount(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }
        DeviceQuery query = new DeviceQuery();
        query.setStoreIds(list.stream().map(StoreVo::getStoreId).collect(Collectors.toList()));
        query.setOnlineStatus(DeviceOnlineStatus.ONLINE.getStatus());
        query.setStatus(DeviceStatus.NORMAL.getStatus());
        query.setGroupBy(DeviceGroupBy.store_id.name());
        Map<Long, Integer> countMap = deviceService.selectCommonCountMap(query, DeviceCountVO::getStoreId);

        for (StoreVo store : list) {
            Integer count = countMap.get(store.getStoreId());
            store.setAvailableDeviceCount(count == null ? 0 : count);
        }
    }

    @Override
    public void assembleOperator(List<StoreVo> list, Long userId) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        for (StoreVo store : list) {
            StoreOperatorVO vo = StoreOperatorVO.defaultInstance();
            // 有操作权限，可修改、配置
            if (storeValidator.canOperaStore(store, userId)) {
                vo.setConfig(true);
                vo.setUpdate(true);
            }
            // 是商户，可删除、查看设备列表
            if (storeValidator.isMch(store, userId)) {
                vo.setDel(true);
                vo.setViewDevice(true);
                vo.setAddStaff(true);
            }
            store.setOperator(vo);
        }
    }

    @Override
    public void assembleBonusList(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        // 查询店铺员工列表
        this.assembleStaffList(list);

        for (StoreVo store : list) {
            List<Bonus> bonusList = bonusConverter.toPoListByVip(store, store.getStaffList(), true);
            store.setBonusList(bonusList);
        }
    }

    @Override
    public void assembleStaffList(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        List<StoreStaffVO> staffList = storeStaffService
                .selectListByStoreIds(CollectionUtils.map(list, StoreVo::getStoreId));

        for (StoreVo store : list) {
            List<StoreStaffVO> storeStaffList = staffList.stream()
                    .filter(staff -> Objects.equals(staff.getStoreId(), store.getStoreId()))
                    .collect(Collectors.toList());
            store.setStaffList(storeStaffList);
        }
    }

    @Override
    public void assembleApply(List<StoreVo> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return;
        }

        StoreApplyQuery query = new StoreApplyQuery();
        query.setIds(CollectionUtils.map(list, StoreVo::getApplyId));
        List<StoreApplyVO> applyList = storeApplyService.selectStoreApplyList(query);

        for (StoreVo store : list) {
            StoreApplyVO apply = applyList.stream()
                    .filter(item -> Objects.equals(item.getId(), store.getApplyId()))
                    .findFirst()
                    .orElse(null);
            store.setApply(apply);
        }
    }
}
