package com.ruoyi.ss.storeType.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.storeType.domain.StoreType;
import com.ruoyi.ss.storeType.domain.StoreTypeQuery;
import com.ruoyi.ss.storeType.domain.StoreTypeVO;
import com.ruoyi.ss.storeType.mapper.StoreTypeMapper;
import com.ruoyi.ss.storeType.service.StoreTypeService;

/**
 * 店铺类型Service业务层处理
 *
 * @author ruoyi
 * @date 2025-02-27
 */
@Service
public class StoreTypeServiceImpl implements StoreTypeService
{
    @Autowired
    private StoreTypeMapper storeTypeMapper;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询店铺类型
     *
     * @param id 店铺类型主键
     * @return 店铺类型
     */
    @Override
    public StoreTypeVO selectStoreTypeById(Long id)
    {
        return storeTypeMapper.selectStoreTypeById(id);
    }

    /**
     * 查询店铺类型列表
     *
     * @param storeType 店铺类型
     * @return 店铺类型
     */
    @Override
    public List<StoreTypeVO> selectStoreTypeList(StoreTypeQuery storeType)
    {
        return storeTypeMapper.selectStoreTypeList(storeType);
    }

    /**
     * 新增店铺类型
     *
     * @param storeType 店铺类型
     * @return 结果
     */
    @Override
    public int insertStoreType(StoreType storeType)
    {
        storeType.setCreateTime(DateUtils.getNowDate());

        Integer result = transactionTemplate.execute(status -> {
            int rows = storeTypeMapper.insertStoreType(storeType);
            if (rows > 0) {
                clearCache();
            }
            return rows;
        });
        return result == null ? 0 : result;
    }

    /**
     * 修改店铺类型
     *
     * @param storeType 店铺类型
     * @return 结果
     */
    @Override
    public int updateStoreType(StoreType storeType)
    {
        Integer result = transactionTemplate.execute(status -> {
            int rows = storeTypeMapper.updateStoreType(storeType);
            if (rows > 0) {
                clearCache();
            }
            return rows;
        });
        return result == null ? 0 : result;
    }

    /**
     * 批量删除店铺类型
     *
     * @param ids 需要删除的店铺类型主键
     * @return 结果
     */
    @Override
    public int deleteStoreTypeByIds(Long[] ids)
    {
        Integer result = transactionTemplate.execute(status -> {
            int rows = storeTypeMapper.deleteStoreTypeByIds(ids);
            if (rows > 0) {
                clearCache();
            }
            return rows;
        });
        return result == null ? 0 : result;
    }

    /**
     * 查询全部店铺类型名称
     * @return
     */
    @Override
    public List<StoreTypeVO> selectListAllName() {
        List<StoreTypeVO> list = redisCache.getCacheObject(CacheConstants.STORE_TYPE_NAME_LIST);
        if (CollectionUtils.isEmptyElement(list)) {
            PageHelper.orderBy("sst.sort asc");
            list = storeTypeMapper.selectStoreTypeList(new StoreTypeQuery());
            if (CollectionUtils.isNotEmptyElement(list)) { 
                redisCache.setCacheObject(CacheConstants.STORE_TYPE_NAME_LIST, list);
            }
        }
        return list;
    }

    private void clearCache() {
        redisCache.deleteObject(CacheConstants.STORE_TYPE_NAME_LIST);
    }
}