package com.ruoyi.ss.transactionBill.domain;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonView;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.constants.DictTypeConstants;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.core.domain.JsonViewProfile;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.ss.account.domain.AccountVO;
import com.ruoyi.system.valid.DictValid;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 充值记录对象 sm_transaction_bill
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class TransactionBill extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 主键 */
    @ApiModelProperty("主键")
    @JsonView(JsonViewProfile.Base.class)
    private Long billId;

    /** 订单编号 */
    @Excel(name = "订单编号", sort = 5)
    @ApiModelProperty("订单编号")
    @JsonView(JsonViewProfile.Base.class)
    private String billNo;

    /** 用户 */
    @ApiModelProperty("用户id")
    private Long userId;

    /** 类型 */
    @ApiModelProperty("类型：1充值，2提现")
    @JsonView(JsonViewProfile.App.class)
    private String type;

    /** 设备 */
    @ApiModelProperty("设备id")
    @JsonView(JsonViewProfile.Base.class)
    private Long deviceId;

    @Excel(name = "设备SN", sort = 20)
    @ApiModelProperty("设备SN")
    @NotNull(message = "设备SN不允许为空", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private String deviceNo;

    /** 商户(到账用户) */
    @ApiModelProperty("商户(到账用户)id")
    @JsonView(JsonViewProfile.App.class)
    private Long mchId;

    /** 交易金额 */
    @Excel(name = "交易金额", sort = 11)
    @ApiModelProperty("交易金额")
    @NotNull(message = "交易金额不允许为空", groups = {ValidGroup.Recharge.class})
    @Min(value = 0, message = "交易金额不允许小于0", groups = {ValidGroup.Recharge.class})
    @Min(value = 20, message = "申请提现金额不允许小于20", groups = {ValidGroup.Withdraw.class})
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal money;

    /** 实际到账金额 */
    @ApiModelProperty("商户实际到账金额")
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal arrivalAmount;

    /** 手续费 */
    @ApiModelProperty("手续费")
    @Min(value = 0, message = "手续费不允许小于0", groups = { ValidGroup.Withdraw.class})
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal serviceCharge;

    @ApiModelProperty("订单状态：1未支付，2支付成功，3已退款，4已取消")
    @Excel(name = "订单状态", dictType = DictTypeConstants.TRANSACTION_BILL_STATUS, sort = 10)
    @JsonView(JsonViewProfile.App.class)
    private String status;

    @ApiModelProperty("渠道id")
    @JsonView(JsonViewProfile.App.class)
    private Long channelId;

    @ApiModelProperty("到账后余额")
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal afterBalance;

    @ApiModelProperty("支付成功的时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonView(JsonViewProfile.App.class)
    @Excel(name = "支付时间", dateFormat = "yyyy-MM-dd HH:mm:ss", sort = 15)
    private Date payTime;

    @ApiModelProperty("支付过期时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expireTime;

    @ApiModelProperty("交易账号（银行卡号、微信openId）")
    @JsonView(JsonViewProfile.App.class)
    private String accountNo;

    @ApiModelProperty("已支付/提现金额")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal payedAmount;

    @ApiModelProperty("微信商家提现明细id列表")
    private List<String> transferIds;

    @ApiModelProperty("渠道成本")
    private BigDecimal channelCost;

    @ApiModelProperty("设备充值状态，0未充值，1充值成功")
    @JsonView(JsonViewProfile.App.class)
    private String deviceRechargeStatus;

    @ApiModelProperty("套餐id")
    @NotNull(message = "套餐不允许为空", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private Long suitId;

    @Excel(name = "套餐时长/电量（秒/度）", sort = 80)
    @ApiModelProperty("套餐时长（秒）")
    @NotNull(message = "套餐时长不允许为空", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private Integer suitTime;

    @Excel(name = "套餐时长单位", dictType = DictTypeConstants.TIME_UNIT, sort = 90)
    @ApiModelProperty("套餐时长单位")
    @NotBlank(message = "套餐时长单位不允许为空", groups = {ValidGroup.Recharge.class})
    @DictValid(type = DictTypeConstants.TIME_UNIT, message = "非法的套餐时长单位", groups = {ValidGroup.Recharge.class})
    @JsonView(JsonViewProfile.App.class)
    private String suitTimeUnit;

    @Excel(name = "套餐开始使用时间", dateFormat = "yyyy-MM-dd HH:mm:ss", sort = 100)
    @ApiModelProperty("套餐开始使用时间")
    @JsonView(JsonViewProfile.App.class)
    private LocalDateTime suitStartTime;

    @Excel(name = "套餐结束使用时间", dateFormat = "yyyy-MM-dd HH:mm:ss", sort = 110)
    @ApiModelProperty("套餐结束使用时间")
    @JsonView(JsonViewProfile.App.class)
    private LocalDateTime suitEndTime;

    @Excel(name = "套餐到期时间", dateFormat = "yyyy-MM-dd HH:mm:ss", sort = 120)
    @ApiModelProperty("套餐到期时间")
    @JsonView(JsonViewProfile.App.class)
    private LocalDateTime suitExpireTime;

    @Excel(name = "套餐开始使用的电量（度）", sort = 130)
    @ApiModelProperty("套餐开始使用的电量")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal suitStartEle;

    @Excel(name = "套餐结束使用的电量（度）", sort = 140)
    @ApiModelProperty("套餐结束使用的电量")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal suitEndEle;

    @Excel(name = "套餐收费模式", dictType = DictTypeConstants.SUIT_FEE_MODE, sort = 150)
    @ApiModelProperty("套餐收费模式")
    @JsonView(JsonViewProfile.App.class)
    private String suitFeeMode;

    @Excel(name = "套餐收费类型", dictType = DictTypeConstants.SUIT_FEE_TYPE, sort = 160)
    @ApiModelProperty("套餐收费类型")
    @JsonView(JsonViewProfile.App.class)
    private String suitFeeType;

    @ApiModelProperty("套餐档位金额列表")
    @JsonView(JsonViewProfile.App.class)
    private List<BigDecimal> suitGearAmount;

    @ApiModelProperty("套餐时段档位列表")
    @JsonView(JsonViewProfile.App.class)
    private List<Integer> suitGearTime;

    @Excel(name = "套餐价格", sort = 170)
    @ApiModelProperty("套餐价格")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal suitPrice;

    @ApiModelProperty("店铺ID")
    @JsonView(JsonViewProfile.App.class)
    private Long storeId;

    @Excel(name = "店铺名称", sort = 180)
    @ApiModelProperty("店铺名称")
    @JsonView(JsonViewProfile.App.class)
    private String storeName;

    @Excel(name = "店铺地址", sort = 190)
    @ApiModelProperty("店铺地址")
    @JsonView(JsonViewProfile.App.class)
    private String storeAddress;

    @Excel(name = "设备名称", sort = 21)
    @ApiModelProperty("设备名称")
    @JsonView(JsonViewProfile.App.class)
    private String deviceName;

    @Excel(name = "套餐名称", sort = 70)
    @ApiModelProperty("套餐名称")
    @JsonView(JsonViewProfile.App.class)
    private String suitName;

    @ApiModelProperty("设备MAC")
    @JsonView(JsonViewProfile.App.class)
    private String deviceMac;

    @ApiModelProperty("设备MAC2")
    @JsonView(JsonViewProfile.App.class)
    private String deviceMac2;

    @ApiModelProperty("设备产品ID")
    private String deviceProductId;

    @ApiModelProperty("设备服务费模式")
    private String deviceServiceMode;

    @Excel(name = "退款金额", sort = 12)
    @ApiModelProperty("退款金额")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal refundAmount;

    @ApiModelProperty("商户退款金额")
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal refundMchAmount;

    @ApiModelProperty("服务费退款金额")
    @JsonView(JsonViewProfile.AppMch.class)
    private BigDecimal refundServiceAmount;

    @ApiModelProperty("支付凭证")
    @JsonView(JsonViewProfile.AppMch.class)
    private String payPicture;

    @ApiModelProperty("提现方式")
    @JsonView(JsonViewProfile.AppMch.class)
    private String withdrawType;

    @ApiModelProperty("提现审核时的二维码")
    private String offlineImage;

    @ApiModelProperty("押金支付成功的支付订单ID")
    private Long depositPayId;

    @ApiModelProperty("支付成功的支付订单ID")
    private Long payId;

    @Excel(name = "低功率自动关闭", readConverterExp = "true=开启,false=关闭", sort = 200)
    @ApiModelProperty("订单是否开启低功率自动关闭")
    private Boolean suitEnableLowPowerClose;

    @Excel(name = "自动关闭阈值(瓦)", sort = 210)
    @ApiModelProperty("自动关闭阈值")
    private BigDecimal suitLowPower;

    @ApiModelProperty("版本号")
    private Integer version;

    @ApiModelProperty("设备关闭状态(1-已关闭，2-未关闭）")
    private String closeStatus;

    @ApiModelProperty("设备关闭状态描述")
    private String closeResult;

    @ApiModelProperty("代理商ID")
    private Long agentId;

    @ApiModelProperty("代理商手机号")
    private String agentMobile;

    @Excel(name = "语音播报", readConverterExp = "true=开启,false=关闭", sort = 220)
    @ApiModelProperty("语音播报")
    @JsonView(JsonViewProfile.App.class)
    private Boolean suitEnabledVoid;

    @Excel(name = "语音播报阈值(分钟)", sort = 230)
    @ApiModelProperty("语音播报阈值")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal suitVoidMinute;

    @ApiModelProperty("套餐语音播报设置结果")
    private String suitVoidResult;

    @ApiModelProperty("套餐语音播报设置结果描述")
    private String suitVoidMsg;

    @Excel(name = "套餐押金（元）", sort = 71)
    @ApiModelProperty("套餐押金")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal suitDeposit;

    @ApiModelProperty("设备开启状态描述")
    private String openMsg;

    @Excel(name = "用户手机号", sort = 240)
    @ApiModelProperty("用户手机号")
    @JsonView(JsonViewProfile.App.class)
    private String userMobile;

    @Excel(name = "展示用户手机号", readConverterExp = "true=展示,false=不展示", sort = 250)
    @ApiModelProperty("展示用户手机号")
    private Boolean mchShowMobile;

    @Excel(name = "手机号价格", sort = 260)
    @ApiModelProperty("手机号价格")
    private BigDecimal mchShowMobilePrice;

    @Excel(name = "展示手机号缴费状态", dictType = DictTypeConstants.RECHARGE_MCH_SHOW_MOBILE_STATUS, sort = 270)
    @ApiModelProperty("展示手机号缴费状态")
    private String mchShowMobileStatus;

    @ApiModelProperty("渠道类型")
    private String channelType;
    
    @ApiModelProperty("使用vip的ID")
    @JsonView(JsonViewProfile.App.class)
    private Long vipId;

    @Excel(name = "会员折扣", sort = 56)
    @ApiModelProperty("会员折扣")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal vipDiscount;

    @Excel(name = "会员名称", sort = 55)
    @ApiModelProperty("会员名称")
    @JsonView(JsonViewProfile.App.class)
    private String vipName;

    @Excel(name = "优惠金额", sort = 13)
    @ApiModelProperty("优惠金额")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal discountAmount;

    @Excel(name = "优惠金额退款", sort = 14)
    @ApiModelProperty("优惠金额退款")
    @JsonView(JsonViewProfile.App.class)
    private BigDecimal discountRefundAmount;

    @ApiModelProperty("提现账户")
    private AccountVO withdrawAccount;

    @ApiModelProperty("是否已经处理过结束流程")
    private Boolean finished;
}
