package com.ruoyi.web.controller.app;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.abnormal.domain.dto.AbnormalSubmitDTO;
import com.ruoyi.ss.abnormal.domain.AbnormalQuery;
import com.ruoyi.ss.abnormal.service.AbnormalConverter;
import com.ruoyi.ss.abnormal.service.AbnormalValidator;
import com.ruoyi.ss.abnormal.service.AbnormalService;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.vipOrder.service.VipOrderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author wjh
 * 2024/4/30
 */
@Api(tags = "设备故障")
@RestController
@RequestMapping("/app/abnormal")
public class AppAbnormalController extends BaseController {

    @Autowired
    private AbnormalService abnormalService;

    @Autowired
    private AbnormalValidator abnormalValidator;

    @Autowired
    private AbnormalConverter abnormalConverter;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private DeviceService deviceService;

    @ApiOperation("提交设备故障反馈")
    @PostMapping("/submit")
    public AjaxResult submit(@RequestBody @Validated(ValidGroup.FrontCreate.class) AbnormalSubmitDTO data) {
        data.setUserId(getUserId());
        data.setCreateBy(getUsername());
        TransactionBillVO transactionBillVO = transactionBillService.selectSmTransactionBillByBillId(data.getOrderId());
        DeviceVO deviceVO = deviceService.selectByDeviceNo(data.getDeviceNo());
        if (data.getOrderId() != null && data.getDeviceNo() != null) {
            if (transactionBillVO == null) {
                return AjaxResult.error("当前订单不存在");
            }
            if (deviceVO == null) {
                return AjaxResult.error("当前设备不存在");
            }
            if (!transactionBillVO.getDeviceNo().equals(data.getDeviceNo())) {
                return AjaxResult.error("当前设备与订单无关联");
            }
        }
        // 获取订单商户id
        if ( data.getOrderId()!= null) {
            data.setOrderNo(transactionBillVO.getBillNo());
            if(transactionBillVO.getMchId()!=null){
                data.setStoreId(transactionBillVO.getMchId());
            }
        }
        // 获取设备商户id
        if (data.getDeviceNo() != null) {
            Long storeId = deviceVO.getStoreId();
            if (storeId != null) {
                data.setStoreId(storeId);
            }
        }
        return success(abnormalService.insertAbnormal(abnormalConverter.toPo(data)));
    }

    @ApiOperation("获取本人提交的故障反馈列表")
    @GetMapping("/list")
    public TableDataInfo list(@Validated(ValidGroup.Query.class) AbnormalQuery query) {
        startPage();
        query.setUserId(getUserId());
        return getDataTable(abnormalService.selectAbnormalList(query));
    }

}
