package com.ruoyi.web.controller.mch;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.dashboard.index.domain.dto.BusinessStatisticsQuery;
import com.ruoyi.dashboard.index.domain.dto.StoreDailyIncomeQuery;
import com.ruoyi.dashboard.index.service.DashboardService;
import com.ruoyi.ss.store.service.StoreValidator;

import io.swagger.annotations.ApiOperation;

/**
 * @author wjh
 * 2024/10/8
 */
@RestController
@RequestMapping("/mch/dashboard")
public class MchDashboardController extends BaseController {

    @Autowired
    private DashboardService dashboardService;

    @Autowired
    private StoreValidator storeValidator;

    @ApiOperation("商户获取店铺维度的订单统计")
    @GetMapping("/businessStatisticsByStore")
    public AjaxResult businessStatisticsByStore(BusinessStatisticsQuery query) {
        query.setMchId(getUserId());
        return success(dashboardService.businessStatisticsByStore(query));
    }

    @ApiOperation("商户获取指定店铺的订单统计")
    @GetMapping("/businessStatisticsByDevice")
    public AjaxResult businessStatisticsByDevice(BusinessStatisticsQuery query) {
        query.setMchId(getUserId());
        return success(dashboardService.businessStatisticsByDevice(query));
    }

    @ApiOperation("获取店铺每日营收数据")
    @GetMapping("/storeDailyIncome")
    public AjaxResult storeDailyIncome(@Validated StoreDailyIncomeQuery query) {
        if (!storeValidator.canViewStoreIncome(query.getStoreId(), getUserId())) {
            return error("您没有权限查看该店铺的营收数据");
        }
        return success(dashboardService.storeDailyIncome(query));
    }

}
