package com.ruoyi.web.controller.mch;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.ss.device.domain.dto.DeviceWxsDTO;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.device.service.DeviceValidator;

import io.swagger.annotations.ApiOperation;

/**
 * @author wjh
 * 2024/7/30
 */
@RestController
@RequestMapping("/mch/device")
public class MchDeviceController extends BaseController {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private DeviceValidator deviceValidator;

    @ApiOperation("商户通过设备IDS查询")
    @GetMapping("/listByIds/{ids}")
    public AjaxResult listByIds(@PathVariable List<Long> ids)
    {
        List<DeviceVO> list = deviceService.selectByIds(ids, getUserId());
        return success(list);
    }

    @ApiOperation("设置电量系数")
    @PutMapping("/setWxs")
    public AjaxResult setWxs(@RequestBody @Validated DeviceWxsDTO dto) {
        DeviceVO device = deviceService.selectById(dto.getDeviceId());
        if (!deviceValidator.canOpera(device, getUserId())) {
            return error("您无权操作ID为" + dto.getDeviceId() + "的设备");
        }
        return toAjax(deviceService.setWxs(device, dto.getWxs(), "商户设置电量系数:" + dto.getWxs()));
    }

}
