package com.ruoyi.web.controller.mch;

import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.device.domain.enums.DevicePowerStatus;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.device.service.DeviceValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 设备操作接口，专门给刷鞋机客户用的（特殊接口）
 * @author wjh
 * 2025/4/16
 */
@RestController
@RequestMapping("/mch/device/iot/sp")
public class MchDeviceIotSpController extends BaseController {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private DeviceValidator deviceValidator;

    @Autowired
    private DeviceAssembler deviceAssembler;

    // 设备开关
    @PutMapping("/switch")
    public AjaxResult switchDevice(Long deviceId, String status) {
        DeviceVO device = deviceService.selectById(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");

        // 判断是否有权限更改
        ServiceUtil.assertion(!deviceValidator.canOpera(deviceId, getUserId()), "您无权操作此设备");
        deviceAssembler.assembleAllowSwitch(device, getUserId());
        ServiceUtil.assertion(!device.getAllowSwitch(), "您无权操作此设备");

        // 获取最新数据
        deviceService.pullDeviceInfo(device, null);
        DevicePowerStatus powerStatus = DevicePowerStatus.parse(status);
        return toAjax(deviceService.switchDevice(device, powerStatus, "刷鞋机商户开关设备", IotConstants.COMMAND_PPPP, IotConstants.COMMAND_CCCCC));
    }

    // 设备增加时间
    @PutMapping("/time")
    public AjaxResult setTime(Long deviceId, Long seconds) {
        DeviceVO device = deviceService.selectById(deviceId);
        ServiceUtil.assertion(device == null, "设备不存在");
        deviceAssembler.assembleAllowSwitch(device, getUserId());
        ServiceUtil.assertion(!device.getAllowSwitch(), "您无权操作此设备");
        return toAjax(deviceService.addTime(deviceId, seconds, "刷鞋机商户手动充值", true));
    }

    // 设备归零
    @PutMapping("/reset")
    public AjaxResult reset(Long deviceId) {
        if (!deviceValidator.canOpera(deviceId, getUserId()) ) {
            return error("您无权操作此设备");
        }
        return toAjax(deviceService.resetWithBill(deviceId, false, null, "刷鞋机商户设备归零", true, false, IotConstants.COMMAND_TTTT));
    }
}
