package com.ruoyi.web.controller.ss;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.bonusRefund.domain.BonusRefund;
import com.ruoyi.ss.bonusRefund.domain.BonusRefundVO;
import com.ruoyi.ss.bonusRefund.domain.BonusRefundQuery;
import com.ruoyi.ss.bonusRefund.service.BonusRefundService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 分成退款记录Controller
 *
 * @author ruoyi
 * @date 2025-01-25
 */
@RestController
@RequestMapping("/ss/bonusRefund")
public class BonusRefundController extends BaseController
{
    @Autowired
    private BonusRefundService bonusRefundService;

    /**
     * 查询分成退款记录列表
     */
    @PreAuthorize("@ss.hasPermi('ss:bonusRefund:list')")
    @GetMapping("/list")
    public TableDataInfo list(BonusRefundQuery query)
    {
        startPage();
        startOrderBy();
        List<BonusRefundVO> list = bonusRefundService.selectBonusRefundList(query);
        return getDataTable(list);
    }

    /**
     * 导出分成退款记录列表
     */
    @PreAuthorize("@ss.hasPermi('ss:bonusRefund:export')")
    @Log(title = "分成退款记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BonusRefundQuery query)
    {
        List<BonusRefundVO> list = bonusRefundService.selectBonusRefundList(query);
        ExcelUtil<BonusRefundVO> util = new ExcelUtil<BonusRefundVO>(BonusRefundVO.class);
        util.exportExcel(response, list, "分成退款记录数据");
    }

    /**
     * 获取分成退款记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:bonusRefund:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(bonusRefundService.selectBonusRefundById(id));
    }

    /**
     * 新增分成退款记录
     */
    @PreAuthorize("@ss.hasPermi('ss:bonusRefund:add')")
    @Log(title = "分成退款记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BonusRefund bonusRefund)
    {
        return toAjax(bonusRefundService.insertBonusRefund(bonusRefund));
    }

    /**
     * 修改分成退款记录
     */
    @PreAuthorize("@ss.hasPermi('ss:bonusRefund:edit')")
    @Log(title = "分成退款记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BonusRefund bonusRefund)
    {
        return toAjax(bonusRefundService.updateBonusRefund(bonusRefund));
    }

    /**
     * 删除分成退款记录
     */
    @PreAuthorize("@ss.hasPermi('ss:bonusRefund:remove')")
    @Log(title = "分成退款记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(bonusRefundService.deleteBonusRefundByIds(ids));
    }
}
