package com.ruoyi.web.controller.ss;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.hardVersion.domain.HardVersion;
import com.ruoyi.ss.hardVersion.domain.HardVersionQuery;
import com.ruoyi.ss.hardVersion.domain.HardVersionVO;
import com.ruoyi.ss.hardVersion.service.HardVersionAssembler;
import com.ruoyi.ss.hardVersion.service.HardVersionService;

/**
 * 硬件版本Controller
 *
 * @author ruoyi
 * @date 2025-03-11
 */
@RestController
@RequestMapping("/ss/hardVersion")
public class HardVersionController extends BaseController
{
    @Autowired
    private HardVersionService hardVersionService;
    
    @Autowired
    private HardVersionAssembler hardVersionAssembler;

    /**
     * 查询硬件版本列表
     */
    @PreAuthorize("@ss.hasPermi('ss:hardVersion:list')")
    @GetMapping("/list")
    public TableDataInfo list(HardVersionQuery query)
    {
        startPage();
        startOrderBy();
        List<HardVersionVO> list = hardVersionService.selectHardVersionList(query);
        hardVersionAssembler.assembleModelList(list);
        return getDataTable(list);
    }

    /**
     * 导出硬件版本列表
     */
    @PreAuthorize("@ss.hasPermi('ss:hardVersion:export')")
    @Log(title = "硬件版本", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HardVersionQuery query)
    {
        List<HardVersionVO> list = hardVersionService.selectHardVersionList(query);
        ExcelUtil<HardVersionVO> util = new ExcelUtil<HardVersionVO>(HardVersionVO.class);
        util.exportExcel(response, list, "硬件版本数据");
    }

    /**
     * 获取硬件版本详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:hardVersion:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(hardVersionService.selectHardVersionById(id));
    }

    /**
     * 新增硬件版本
     */
    @PreAuthorize("@ss.hasPermi('ss:hardVersion:add')")
    @Log(title = "硬件版本", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HardVersion hardVersion)
    {
        return toAjax(hardVersionService.insertHardVersion(hardVersion));
    }

    /**
     * 修改硬件版本
     */
    @PreAuthorize("@ss.hasPermi('ss:hardVersion:edit')")
    @Log(title = "硬件版本", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HardVersion hardVersion)
    {
        return toAjax(hardVersionService.updateHardVersion(hardVersion));
    }

    /**
     * 删除硬件版本
     */
    @PreAuthorize("@ss.hasPermi('ss:hardVersion:remove')")
    @Log(title = "硬件版本", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(hardVersionService.deleteHardVersionByIds(ids));
    }
}
