package com.ruoyi.web.controller.ss;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.help.domain.Help;
import com.ruoyi.ss.help.domain.HelpQuery;
import com.ruoyi.ss.help.domain.HelpVO;
import com.ruoyi.ss.help.service.HelpService;

/**
 * 教程Controller
 *
 * @author ruoyi
 * @date 2025-02-26
 */
@RestController
@RequestMapping("/ss/help")
public class HelpController extends BaseController
{
    @Autowired
    private HelpService helpService;

    /**
     * 查询教程列表
     */
    @PreAuthorize("@ss.hasPermi('ss:help:list')")
    @GetMapping("/list")
    public TableDataInfo list(HelpQuery query)
    {
        startPage();
        startOrderBy();
        List<HelpVO> list = helpService.selectHelpList(query);
        return getDataTable(list);
    }

    /**
     * 导出教程列表
     */
    @PreAuthorize("@ss.hasPermi('ss:help:export')")
    @Log(title = "教程", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HelpQuery query)
    {
        List<HelpVO> list = helpService.selectHelpList(query);
        ExcelUtil<HelpVO> util = new ExcelUtil<HelpVO>(HelpVO.class);
        util.exportExcel(response, list, "教程数据");
    }

    /**
     * 获取教程详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:help:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(helpService.selectHelpById(id));
    }

    /**
     * 新增教程
     */
    @PreAuthorize("@ss.hasPermi('ss:help:add')")
    @Log(title = "教程", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) Help help)
    {
        return toAjax(helpService.insertHelp(help));
    }

    /**
     * 修改教程
     */
    @PreAuthorize("@ss.hasPermi('ss:help:edit')")
    @Log(title = "教程", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) Help help)
    {
        return toAjax(helpService.updateHelp(help));
    }

    /**
     * 删除教程
     */
    @PreAuthorize("@ss.hasPermi('ss:help:remove')")
    @Log(title = "教程", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(helpService.deleteHelpByIds(ids));
    }
}
