package com.ruoyi.web.controller.ss;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.receiveBill.domain.ReceiveBill;
import com.ruoyi.ss.receiveBill.domain.ReceiveBillVO;
import com.ruoyi.ss.receiveBill.domain.ReceiveBillQuery;
import com.ruoyi.ss.receiveBill.service.ReceiveBillService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 应收账单Controller
 *
 * @author ruoyi
 * @date 2024-07-22
 */
@RestController
@RequestMapping("/ss/receiveBill")
public class ReceiveBillController extends BaseController
{
    @Autowired
    private ReceiveBillService receiveBillService;

    /**
     * 查询应收账单列表
     */
    @PreAuthorize("@ss.hasPermi('ss:receiveBill:list')")
    @GetMapping("/list")
    public TableDataInfo list(ReceiveBillQuery query)
    {
        startPage();
        startOrderBy();
        List<ReceiveBillVO> list = receiveBillService.selectReceiveBillList(query);
        return getDataTable(list);
    }

    /**
     * 导出应收账单列表
     */
    @PreAuthorize("@ss.hasPermi('ss:receiveBill:export')")
    @Log(title = "应收账单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReceiveBillQuery query)
    {
        List<ReceiveBillVO> list = receiveBillService.selectReceiveBillList(query);
        ExcelUtil<ReceiveBillVO> util = new ExcelUtil<ReceiveBillVO>(ReceiveBillVO.class);
        util.exportExcel(response, list, "应收账单数据");
    }

    /**
     * 获取应收账单详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:receiveBill:query')")
    @GetMapping(value = "/{billId}")
    public AjaxResult getInfo(@PathVariable("billId") Long billId)
    {
        return success(receiveBillService.selectReceiveBillByBillId(billId));
    }

    /**
     * 新增应收账单
     */
    @PreAuthorize("@ss.hasPermi('ss:receiveBill:add')")
    @Log(title = "应收账单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ReceiveBill receiveBill)
    {
        return toAjax(receiveBillService.insertReceiveBill(receiveBill));
    }

    /**
     * 修改应收账单
     */
    @PreAuthorize("@ss.hasPermi('ss:receiveBill:edit')")
    @Log(title = "应收账单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ReceiveBill receiveBill)
    {
        return toAjax(receiveBillService.updateReceiveBill(receiveBill));
    }

    /**
     * 删除应收账单
     */
    @PreAuthorize("@ss.hasPermi('ss:receiveBill:remove')")
    @Log(title = "应收账单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{billIds}")
    public AjaxResult remove(@PathVariable Long[] billIds)
    {
        return toAjax(receiveBillService.deleteReceiveBillByBillIds(billIds));
    }
}
