package com.ruoyi.web.controller.ss;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.reconciliationDate.domain.ReconciliationDateQuery;
import com.ruoyi.ss.reconciliationDate.domain.ReconciliationDateVO;
import com.ruoyi.ss.reconciliationDate.service.ReconciliationDateService;

/**
 * 日期对账Controller
 *
 * @author ruoyi
 * @date 2025-01-25
 */
@RestController
@RequestMapping("/ss/reconciliationDate")
public class ReconciliationDateController extends BaseController
{
    @Autowired
    private ReconciliationDateService reconciliationDateService;

    /**
     * 查询日期对账列表
     */
    @PreAuthorize("@ss.hasPermi('ss:reconciliationDate:list')")
    @GetMapping("/list")
    public TableDataInfo list(ReconciliationDateQuery query)
    {
        startPage();
        startOrderBy();
        List<ReconciliationDateVO> list = reconciliationDateService.selectReconciliationDateList(query);
        return getDataTable(list);
    }

    @GetMapping("/listGroupByDate")
    public TableDataInfo listGroupByDate(ReconciliationDateQuery query) {
        startPage();
        startOrderBy();
        List<ReconciliationDateVO> list = reconciliationDateService.selectReconciliationDateListGroupBy(query, "date");
        return getDataTable(list);
    }

    @GetMapping("/listGroupByChannel")
    public TableDataInfo listGroupByChannel(ReconciliationDateQuery query) {
        startPage();
        startOrderBy();
        List<ReconciliationDateVO> list = reconciliationDateService.selectReconciliationDateListGroupBy(query, "date, channel_type, channel_id");
        return getDataTable(list);
    }

    /**
     * 导出日期对账列表
     */
    @PreAuthorize("@ss.hasPermi('ss:reconciliationDate:export')")
    @Log(title = "日期对账", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ReconciliationDateQuery query)
    {
        List<ReconciliationDateVO> list = reconciliationDateService.selectReconciliationDateList(query);
        ExcelUtil<ReconciliationDateVO> util = new ExcelUtil<ReconciliationDateVO>(ReconciliationDateVO.class);
        util.exportExcel(response, list, "日期对账数据");
    }

    /**
     * 获取日期对账详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:reconciliationDate:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(reconciliationDateService.selectReconciliationDateById(id));
    }

    /**
     * 新增日期对账
     */
    // @PreAuthorize("@ss.hasPermi('ss:reconciliationDate:add')")
    // @Log(title = "日期对账", businessType = BusinessType.INSERT)
    // @PostMapping
    // public AjaxResult add(@RequestBody ReconciliationDate reconciliationDate)
    // {
    //     return toAjax(reconciliationDateService.insertReconciliationDate(reconciliationDate));
    // }

    // /**
    //  * 修改日期对账
    //  */
    // @PreAuthorize("@ss.hasPermi('ss:reconciliationDate:edit')")
    // @Log(title = "日期对账", businessType = BusinessType.UPDATE)
    // @PutMapping
    // public AjaxResult edit(@RequestBody ReconciliationDate reconciliationDate)
    // {
    //     return toAjax(reconciliationDateService.updateReconciliationDate(reconciliationDate));
    // }

    // /**
    //  * 删除日期对账
    //  */
    // @PreAuthorize("@ss.hasPermi('ss:reconciliationDate:remove')")
    // @Log(title = "日期对账", businessType = BusinessType.DELETE)
	// @DeleteMapping("/{ids}")
    // public AjaxResult remove(@PathVariable Long[] ids)
    // {
    //     return toAjax(reconciliationDateService.deleteReconciliationDateByIds(ids));
    // }
}
