package com.ruoyi.web.controller.ss;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.snLog.domain.SnLog;
import com.ruoyi.ss.snLog.domain.SnLogQuery;
import com.ruoyi.ss.snLog.domain.SnLogVO;
import com.ruoyi.ss.snLog.service.SnLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * Sn码更换日志Controller
 *
 * @author ruoyi
 * @date 2025-03-31
 */
@RestController
@RequestMapping("/ss/snLog")
public class SnLogController extends BaseController
{
    @Autowired
    private SnLogService snLogService;

    /**
     * 查询Sn码更换日志列表
     */
    @PreAuthorize("@ss.hasPermi('ss:snLog:list')")
    @GetMapping("/list")
    public TableDataInfo list(SnLogQuery query)
    {
        startPage();
        startOrderBy();
        List<SnLogVO> list = snLogService.selectSnLogList(query);
        return getDataTable(list);
    }

    /**
     * 导出Sn码更换日志列表
     */
    @PreAuthorize("@ss.hasPermi('ss:snLog:export')")
    @Log(title = "Sn码更换日志", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SnLogQuery query)
    {
        List<SnLogVO> list = snLogService.selectSnLogList(query);
        ExcelUtil<SnLogVO> util = new ExcelUtil<SnLogVO>(SnLogVO.class);
        util.exportExcel(response, list, "Sn码更换日志数据");
    }

    /**
     * 获取Sn码更换日志详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:snLog:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(snLogService.selectSnLogById(id));
    }

    /**
     * 新增Sn码更换日志
     */
    @PreAuthorize("@ss.hasPermi('ss:snLog:add')")
    @Log(title = "Sn码更换日志", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SnLog snLog)
    {
        return toAjax(snLogService.insertSnLog(snLog));
    }

    /**
     * 修改Sn码更换日志
     */
    @PreAuthorize("@ss.hasPermi('ss:snLog:edit')")
    @Log(title = "Sn码更换日志", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SnLog snLog)
    {
        return toAjax(snLogService.updateSnLog(snLog));
    }

    /**
     * 删除Sn码更换日志
     */
    @PreAuthorize("@ss.hasPermi('ss:snLog:remove')")
    @Log(title = "Sn码更换日志", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(snLogService.deleteSnLogByIds(ids));
    }
}
