package com.ruoyi.web.controller.ss;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.ss.storeType.domain.StoreType;
import com.ruoyi.ss.storeType.domain.StoreTypeQuery;
import com.ruoyi.ss.storeType.domain.StoreTypeVO;
import com.ruoyi.ss.storeType.service.StoreTypeService;

/**
 * 店铺类型Controller
 *
 * @author ruoyi
 * @date 2025-02-27
 */
@RestController
@RequestMapping("/ss/storeType")
public class StoreTypeController extends BaseController
{
    @Autowired
    private StoreTypeService storeTypeService;

    /**
     * 查询店铺类型列表
     */
    @PreAuthorize("@ss.hasPermi('ss:storeType:list')")
    @GetMapping("/list")
    public TableDataInfo list(StoreTypeQuery query)
    {
        startPage();
        startOrderBy();
        List<StoreTypeVO> list = storeTypeService.selectStoreTypeList(query);
        return getDataTable(list);
    }

    /**
     * 查询全部店铺类型
     * @return
     */
    @GetMapping("/listAll")
    public AjaxResult listAll() {
        return success(storeTypeService.selectListAllName());
    }

    /**
     * 导出店铺类型列表
     */
    @PreAuthorize("@ss.hasPermi('ss:storeType:export')")
    @Log(title = "店铺类型", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, StoreTypeQuery query)
    {
        List<StoreTypeVO> list = storeTypeService.selectStoreTypeList(query);
        ExcelUtil<StoreTypeVO> util = new ExcelUtil<StoreTypeVO>(StoreTypeVO.class);
        util.exportExcel(response, list, "店铺类型数据");
    }

    /**
     * 获取店铺类型详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:storeType:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(storeTypeService.selectStoreTypeById(id));
    }

    /**
     * 新增店铺类型
     */
    @PreAuthorize("@ss.hasPermi('ss:storeType:add')")
    @Log(title = "店铺类型", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated(ValidGroup.Create.class) StoreType storeType)
    {
        return toAjax(storeTypeService.insertStoreType(storeType));
    }

    /**
     * 修改店铺类型
     */
    @PreAuthorize("@ss.hasPermi('ss:storeType:edit')")
    @Log(title = "店铺类型", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated(ValidGroup.Update.class) StoreType storeType)
    {
        return toAjax(storeTypeService.updateStoreType(storeType));
    }

    /**
     * 删除店铺类型
     */
    @PreAuthorize("@ss.hasPermi('ss:storeType:remove')")
    @Log(title = "店铺类型", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(storeTypeService.deleteStoreTypeByIds(ids));
    }
}
